import unittest

import reactivex
from reactivex.testing import ReactiveTest, TestScheduler

on_next = ReactiveTest.on_next
on_completed = ReactiveTest.on_completed
on_error = ReactiveTest.on_error
subscribe = ReactiveTest.subscribe
subscribed = ReactiveTest.subscribed
disposed = ReactiveTest.disposed
created = ReactiveTest.created


class RxException(Exception):
    pass


# Helper function for raising exceptions within lambdas
def _raise(ex):
    raise RxException(ex)


class TestTimer(unittest.TestCase):
    def test_oneshot_timer_date_basic(self):
        scheduler = TestScheduler()
        date = scheduler.to_datetime(250.0)

        def create():
            return reactivex.timer(duetime=date)

        results = scheduler.start(create)
        assert results.messages == [on_next(250.0, 0), on_completed(250.0)]

    def test_oneshot_timer_date_passed(self):
        scheduler = TestScheduler()
        date = scheduler.to_datetime(90.0)

        def create():
            return reactivex.timer(date)

        results = scheduler.start(create)
        assert results.messages == [on_next(200, 0), on_completed(200)]

    def test_oneshot_timer_date_disposed(self):
        scheduler = TestScheduler()
        date = scheduler.to_datetime(1010.0)

        def create():
            return reactivex.timer(date)

        results = scheduler.start(create)
        assert results.messages == []

    def test_oneshot_timer_date_observer_throws(self):
        scheduler = TestScheduler()
        date = scheduler.to_datetime(250.0)
        xs = reactivex.timer(date)
        xs.subscribe(lambda x: _raise("ex"), scheduler=scheduler)

        self.assertRaises(RxException, scheduler.start)

    def test_oneshot_timer_timespan_basic(self):
        scheduler = TestScheduler()

        def create():
            return reactivex.timer(duetime=300)

        results = scheduler.start(create)
        assert results.messages == [on_next(500, 0), on_completed(500)]

    def test_oneshot_timer_timespan_zero(self):
        scheduler = TestScheduler()

        def create():
            return reactivex.timer(0)

        results = scheduler.start(create)
        assert results.messages == [on_next(200, 0), on_completed(200)]

    def test_oneshot_timer_timespan_negative(self):
        scheduler = TestScheduler()

        def create():
            return reactivex.timer(-1)

        results = scheduler.start(create)
        assert results.messages == [on_next(200, 0), on_completed(200)]

    def test_oneshot_timer_timespan_disposed(self):
        scheduler = TestScheduler()

        def create():
            return reactivex.timer(1000)

        results = scheduler.start(create)
        assert results.messages == []

    def test_oneshot_timer_timespan_observer_throws(self):
        scheduler1 = TestScheduler()
        xs = reactivex.timer(11)
        xs.subscribe(lambda x: _raise("ex"), scheduler=scheduler1)

        self.assertRaises(RxException, scheduler1.start)

        scheduler2 = TestScheduler()
        ys = reactivex.timer(1, period=None)
        ys.subscribe(on_completed=lambda: _raise("ex"), scheduler=scheduler2)

        self.assertRaises(RxException, scheduler2.start)

    def test_periodic_timer_basic(self):
        scheduler = TestScheduler()

        def create():
            return reactivex.timer(duetime=300, period=400)

        results = scheduler.start(create)
        assert results.messages == [on_next(500, 0), on_next(900, 1)]

    def test_periodic_timer_equal_time_and_period(self):
        scheduler = TestScheduler()

        def create():
            return reactivex.timer(duetime=300, period=300)

        results = scheduler.start(create)
        assert results.messages == [on_next(500, 0), on_next(800, 1)]



import unittest

from reactivex import operators as ops
from reactivex.testing import ReactiveTest, TestScheduler

on_next = ReactiveTest.on_next
on_completed = ReactiveTest.on_completed
on_error = ReactiveTest.on_error
subscribe = ReactiveTest.subscribe
subscribed = ReactiveTest.subscribed
disposed = ReactiveTest.disposed
created = ReactiveTest.created


class TestContains(unittest.TestCase):
    def test_contains_empty(self):
        scheduler = TestScheduler()
        msgs = [on_next(150, 1), on_completed(250)]
        xs = scheduler.create_hot_observable(msgs)

        def create():
            return xs.pipe(ops.contains(42))

        res = scheduler.start(create=create).messages
        assert res == [on_next(250, False), on_completed(250)]

    def test_contains_return_positive(self):
        scheduler = TestScheduler()
        msgs = [on_next(150, 1), on_next(210, 2), on_completed(250)]
        xs = scheduler.create_hot_observable(msgs)

        def create():
            return xs.pipe(ops.contains(2))

        res = scheduler.start(create=create).messages
        assert res == [on_next(210, True), on_completed(210)]

    def test_contains_return_negative(self):
        scheduler = TestScheduler()
        msgs = [on_next(150, 1), on_next(210, 2), on_completed(250)]
        xs = scheduler.create_hot_observable(msgs)

        def create():
            return xs.pipe(ops.contains(-2))

        res = scheduler.start(create=create).messages
        assert res == [on_next(250, False), on_completed(250)]

    def test_contains_some_positive(self):
        scheduler = TestScheduler()
        msgs = [
            on_next(150, 1),
            on_next(210, 2),
            on_next(220, 3),
            on_next(230, 4),
            on_completed(250),
        ]
        xs = scheduler.create_hot_observable(msgs)

        def create():
            return xs.pipe(ops.contains(3))

        res = scheduler.start(create=create).messages
        assert res == [on_next(220, True), on_completed(220)]

    def test_contains_some_negative(self):
        scheduler = TestScheduler()
        msgs = [
            on_next(150, 1),
            on_next(210, 2),
            on_next(220, 3),
            on_next(230, 4),
            on_completed(250),
        ]
        xs = scheduler.create_hot_observable(msgs)

        def create():
            return xs.pipe(ops.contains(-3))

        res = scheduler.start(create=create).messages
        assert res == [on_next(250, False), on_completed(250)]

    def test_contains_on_error(self):
        ex = "ex"
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(on_next(150, 1), on_error(210, ex))

        def create():
            return xs.pipe(ops.contains(42))

        res = scheduler.start(create=create).messages
        assert res == [on_error(210, ex)]

    def test_contains_never(self):
        scheduler = TestScheduler()
        msgs = [on_next(150, 1)]
        xs = scheduler.create_hot_observable(msgs)

        def create():
            return xs.pipe(ops.contains(42))

        res = scheduler.start(create=create).messages
        assert res == []

    def test_contains_comparer_throws(self):
        ex = "ex"
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(on_next(150, 1), on_next(210, 2))

        def create():
            def comparer(a, b):
                raise Exception(ex)

            return xs.pipe(ops.contains(42, comparer))

        res = scheduler.start(create=create).messages
        assert res == [on_error(210, ex)]

    def test_contains_comparer_contains_value(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(150, 1),
            on_next(210, 3),
            on_next(220, 4),
            on_next(230, 8),
            on_completed(250),
        )

        def create():
            return xs.pipe(ops.contains(42, lambda a, b: a % 2 == b % 2))

        res = scheduler.start(create=create).messages
        assert res == [on_next(220, True), on_completed(220)]

    def test_contains_comparer_does_not_contain_value(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(150, 1),
            on_next(210, 2),
            on_next(220, 4),
            on_next(230, 8),
            on_completed(250),
        )

        def create():
            return xs.pipe(ops.contains(21, lambda a, b: a % 2 == b % 2))

        res = scheduler.start(create=create).messages
        assert res == [on_next(250, False), on_completed(250)]



import unittest

import reactivex
from reactivex import ConnectableObservable, Observable
from reactivex import operators as ops
from reactivex.abc import ObserverBase
from reactivex.subject import Subject
from reactivex.testing import ReactiveTest, TestScheduler

on_next = ReactiveTest.on_next
on_completed = ReactiveTest.on_completed
on_error = ReactiveTest.on_error
subscribe = ReactiveTest.subscribe
subscribed = ReactiveTest.subscribed
disposed = ReactiveTest.disposed
created = ReactiveTest.created


class MySubject(Observable, ObserverBase):
    def __init__(self):

        super(MySubject, self).__init__()

        self.dispose_on_map = {}
        self.subscribe_count = 0
        self.disposed = False

    def _subscribe_core(self, observer, scheduler=None):
        self.subscribe_count += 1
        self.observer = observer

        class Duck:
            def __init__(self, this):
                self.this = this

            def dispose(self) -> None:
                self.this.disposed = True

        return Duck(self)

    def dispose_on(self, value, disposable):
        self.dispose_on_map[value] = disposable

    def on_next(self, value):
        self.observer.on_next(value)
        if value in self.dispose_on_map:
            self.dispose_on_map[value].dispose()

    def on_error(self, error):
        self.observer.on_error(error)

    def on_completed(self):
        self.observer.on_completed()


class TestConnectableObservable(unittest.TestCase):
    def test_connectable_observable_creation(self):
        y = [0]

        s2 = Subject()
        co2 = ConnectableObservable(reactivex.return_value(1), s2)

        def on_next(x):
            y[0] = x

        co2.subscribe(on_next=on_next)
        self.assertNotEqual(1, y[0])

        co2.connect()
        self.assertEqual(1, y[0])

    def test_connectable_observable_connected(self):
        scheduler = TestScheduler()

        xs = scheduler.create_hot_observable(
            on_next(210, 1),
            on_next(220, 2),
            on_next(230, 3),
            on_next(240, 4),
            on_completed(250),
        )

        subject = MySubject()

        conn = ConnectableObservable(xs, subject)
        disconnect = conn.connect(scheduler)

        res = scheduler.start(lambda: conn)

        assert res.messages == [
            on_next(210, 1),
            on_next(220, 2),
            on_next(230, 3),
            on_next(240, 4),
            on_completed(250),
        ]

    def test_connectable_observable_not_connected(self):
        scheduler = TestScheduler()

        xs = scheduler.create_hot_observable(
            on_next(210, 1),
            on_next(220, 2),
            on_next(230, 3),
            on_next(240, 4),
            on_completed(250),
        )

        subject = MySubject()

        conn = ConnectableObservable(xs, subject)

        res = scheduler.start(lambda: conn)

        assert res.messages == []

    def test_connectable_observable_disconnected(self):
        scheduler = TestScheduler()

        xs = scheduler.create_hot_observable(
            on_next(210, 1),
            on_next(220, 2),
            on_next(230, 3),
            on_next(240, 4),
            on_completed(250),
        )

        subject = MySubject()

        conn = ConnectableObservable(xs, subject)
        disconnect = conn.connect(scheduler)
        disconnect.dispose()

        res = scheduler.start(lambda: conn)

        assert res.messages == []

    def test_connectable_observable_disconnect_future(self):
        scheduler = TestScheduler()

        xs = scheduler.create_hot_observable(
            on_next(210, 1),
            on_next(220, 2),
            on_next(230, 3),
            on_next(240, 4),
            on_completed(250),
        )

        subject = MySubject()

        conn = ConnectableObservable(xs, subject)
        subject.dispose_on(3, conn.connect())

        res = scheduler.start(lambda: conn)

        assert res.messages == [on_next(210, 1), on_next(220, 2), on_next(230, 3)]

    def test_connectable_observable_multiple_non_overlapped_connections(self):
        scheduler = TestScheduler()

        xs = scheduler.create_hot_observable(
            on_next(210, 1),
            on_next(220, 2),
            on_next(230, 3),
            on_next(240, 4),
            on_next(250, 5),
            on_next(260, 6),
            on_next(270, 7),
            on_next(280, 8),
            on_next(290, 9),
            on_completed(300),
        )

        subject = Subject()

        conn = xs.pipe(ops.multicast(subject))

        c1 = [None]

        def action10(scheduler, state):
            c1[0] = conn.connect(scheduler)

        scheduler.schedule_absolute(225, action10)

        def action11(scheduler, state):
            c1[0].dispose()

        scheduler.schedule_absolute(241, action11)

        def action12(scheduler, state):
            c1[0].dispose()  # idempotency test

        scheduler.schedule_absolute(245, action12)

        def action13(scheduler, state):
            c1[0].dispose()  # idempotency test

        scheduler.schedule_absolute(251, action13)

        def action14(scheduler, state):
            c1[0].dispose()  # idempotency test

        scheduler.schedule_absolute(260, action14)

        c2 = [None]

        def action20(scheduler, state):
            c2[0] = conn.connect(scheduler)

        scheduler.schedule_absolute(249, action20)

        def action21(scheduler, state):
            c2[0].dispose()

        scheduler.schedule_absolute(255, action21)

        def action22(scheduler, state):
            c2[0].dispose()  # idempotency test

        scheduler.schedule_absolute(265, action22)

        def action23(scheduler, state):
            c2[0].dispose()  # idempotency test

        scheduler.schedule_absolute(280, action23)

        c3 = [None]

        def action30(scheduler, state):
            c3[0] = conn.connect(scheduler)

        scheduler.schedule_absolute(275, action30)

        def action31(scheduler, state):
            c3[0].dispose()

        scheduler.schedule_absolute(295, action31)

        res = scheduler.start(lambda: conn)

        assert res.messages == [
            on_next(230, 3),
            on_next(240, 4),
            on_next(250, 5),
            on_next(280, 8),
            on_next(290, 9),
        ]

        assert xs.subscriptions == [
            subscribe(225, 241),
            subscribe(249, 255),
            subscribe(275, 295),
        ]

    def test_connectable_observable_forward_scheduler(self):
        scheduler = TestScheduler()
        subscribe_scheduler = "unknown"

        def subscribe(observer, scheduler=None):
            nonlocal subscribe_scheduler
            subscribe_scheduler = scheduler

        xs = reactivex.create(subscribe)
        subject = MySubject()

        conn = ConnectableObservable(xs, subject)
        conn.connect(scheduler)

        assert subscribe_scheduler is scheduler



import asyncio
from asyncio import Future

import reactivex
from reactivex import Observable
from reactivex import operators as ops
from reactivex.scheduler.eventloop import AsyncIOScheduler


def to_async_iterable():
    def _to_async_iterable(source: Observable):
        class AIterable:
            def __aiter__(self):
                class AIterator:
                    def __init__(self):
                        self.notifications = []
                        self.future = Future()

                        source.pipe(ops.materialize()).subscribe(self.on_next)

                    def feeder(self):
                        if not self.notifications or self.future.done():
                            return

                        notification = self.notifications.pop(0)
                        dispatch = {
                            "N": lambda: self.future.set_result(notification.value),
                            "E": lambda: self.future.set_exception(
                                notification.exception
                            ),
                            "C": lambda: self.future.set_exception(StopAsyncIteration),
                        }

                        dispatch[notification.kind]()

                    def on_next(self, notification):
                        self.notifications.append(notification)
                        self.feeder()

                    async def __anext__(self):
                        self.feeder()

                        value = await self.future
                        self.future = Future()
                        return value

                return AIterator()

        return AIterable()

    return _to_async_iterable


async def go(loop):
    scheduler = AsyncIOScheduler(loop)

    ai = reactivex.range(0, 10, scheduler=scheduler).pipe(to_async_iterable())
    async for x in ai:
        print("got %s" % x)


def main():
    loop = asyncio.get_event_loop()
    loop.run_until_complete(go(loop))


if __name__ == "__main__":
    main()



import os
import unittest
from datetime import timedelta
from time import sleep

import pytest

from reactivex.internal.basic import default_now
from reactivex.scheduler.mainloop import TkinterScheduler

tkinter = pytest.importorskip("tkinter")

try:
    root = tkinter.Tk()
    root.withdraw()  # Don't actually draw anything
    display = True
except Exception:
    display = False

CI = os.getenv("CI") is not None


@pytest.mark.skipif("display == False")
class TestTkinterScheduler(unittest.TestCase):
    @pytest.mark.skipif(CI, reason="Flaky test in GitHub Actions")
    def test_tkinter_schedule_now(self):
        scheduler = TkinterScheduler(root)
        res = scheduler.now - default_now()
        assert abs(res) <= timedelta(milliseconds=1)

    @pytest.mark.skipif(CI, reason="Flaky test in GitHub Actions")
    def test_tkinter_schedule_now_units(self):
        scheduler = TkinterScheduler(root)
        diff = scheduler.now
        sleep(1.1)
        diff = scheduler.now - diff
        assert timedelta(milliseconds=1000) < diff < timedelta(milliseconds=1300)

    def test_tkinter_schedule_action(self):
        scheduler = TkinterScheduler(root)
        ran = False

        def action(scheduler, state):
            nonlocal ran
            ran = True

        scheduler.schedule(action)

        def done():
            assert ran is True
            root.quit()

        root.after_idle(done)
        root.mainloop()

    def test_tkinter_schedule_action_due(self):
        scheduler = TkinterScheduler(root)
        starttime = default_now()
        endtime = None

        def action(scheduler, state):
            nonlocal endtime
            endtime = default_now()

        scheduler.schedule_relative(0.2, action)

        def done():
            root.quit()
            assert endtime is not None
            diff = endtime - starttime
            assert diff > timedelta(milliseconds=180)

        root.after(300, done)
        root.mainloop()

    def test_tkinter_schedule_action_cancel(self):
        ran = False
        scheduler = TkinterScheduler(root)

        def action(scheduler, state):
            nonlocal ran
            ran = True

        d = scheduler.schedule_relative(0.1, action)
        d.dispose()

        def done():
            root.quit()
            assert ran is False

        root.after(300, done)
        root.mainloop()

    def test_tkinter_schedule_action_periodic(self):
        scheduler = TkinterScheduler(root)
        period = 0.050
        counter = 3

        def action(state):
            nonlocal counter
            if state:
                counter -= 1
                return state - 1

        scheduler.schedule_periodic(period, action, counter)

        def done():
            root.quit()
            assert counter == 0

        root.after(300, done)
        root.mainloop()



import unittest

from reactivex import Observable, never
from reactivex import operators as _
from reactivex.testing import ReactiveTest, TestScheduler

on_next = ReactiveTest.on_next
on_completed = ReactiveTest.on_completed
on_error = ReactiveTest.on_error
subscribe = ReactiveTest.subscribe
subscribed = ReactiveTest.subscribed
disposed = ReactiveTest.disposed
created = ReactiveTest.created


class TestScan(unittest.TestCase):
    def test_scan_seed_never(self):
        scheduler = TestScheduler()
        seed = 42

        def create():
            def func(acc: int, x: int) -> int:
                return acc + x

            return never().pipe(_.scan(seed=seed, accumulator=func))

        results = scheduler.start(create)
        assert results.messages == []

    def test_scan_seed_empty(self):
        scheduler = TestScheduler()
        seed = 42
        xs = scheduler.create_hot_observable(on_next(150, 1), on_completed(250))

        def create():
            return xs.pipe(_.scan(lambda acc, x: acc + x, seed=seed))

        results = scheduler.start(create).messages
        assert len(results) == 1
        assert results[0].value.kind == "C" and results[0].time == 250

    def test_scan_seed_return(self):
        scheduler = TestScheduler()
        seed = 42
        xs = scheduler.create_hot_observable(
            on_next(150, 1), on_next(220, 2), on_completed(250)
        )

        def create():
            return xs.pipe(_.scan(lambda acc, x: acc + x, seed=seed))

        results = scheduler.start(create).messages
        assert len(results) == 2
        assert (
            results[0].value.kind == "N"
            and results[0].value.value == seed + 2
            and results[0].time == 220
        )
        assert results[1].value.kind == "C" and results[1].time == 250

    def test_scan_seed_on_error(self):
        ex = "ex"
        scheduler = TestScheduler()
        seed = 42
        xs = scheduler.create_hot_observable(on_next(150, 1), on_error(250, ex))

        def create() -> Observable[int]:
            return xs.pipe(_.scan(lambda acc, x: acc + x, seed))

        results = scheduler.start(create).messages
        assert len(results) == 1
        assert (
            results[0].value.kind == "E"
            and str(results[0].value.exception) == ex
            and results[0].time == 250
        )

    def test_scan_seed_somedata(self):
        scheduler = TestScheduler()
        seed = 1
        xs = scheduler.create_hot_observable(
            on_next(150, 1),
            on_next(210, 2),
            on_next(220, 3),
            on_next(230, 4),
            on_next(240, 5),
            on_completed(250),
        )

        def create():
            return xs.pipe(_.scan(lambda acc, x: acc + x, seed=seed))

        results = scheduler.start(create).messages
        assert len(results) == 5
        assert (
            results[0].value.kind == "N"
            and results[0].value.value == seed + 2
            and results[0].time == 210
        )
        assert (
            results[1].value.kind == "N"
            and results[1].value.value == seed + 2 + 3
            and results[1].time == 220
        )
        assert (
            results[2].value.kind == "N"
            and results[2].value.value == seed + 2 + 3 + 4
            and results[2].time == 230
        )
        assert (
            results[3].value.kind == "N"
            and results[3].value.value == seed + 2 + 3 + 4 + 5
            and results[3].time == 240
        )
        assert results[4].value.kind == "C" and results[4].time == 250

    def test_scan_noseed_never(self):
        scheduler = TestScheduler()

        def create() -> Observable[int]:
            return never().pipe(_.scan(lambda acc, x: acc + x))

        results = scheduler.start(create)
        assert results.messages == []

    def test_scan_noseed_empty(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(on_next(150, 1), on_completed(250))

        def create() -> Observable[int]:
            return xs.pipe(_.scan(lambda acc, x: acc + x))

        results = scheduler.start(create).messages
        assert len(results) == 1
        assert results[0].value.kind == "C" and results[0].time == 250

    def test_scan_noseed_return(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(150, 1), on_next(220, 2), on_completed(250)
        )

        def create() -> Observable[int]:
            def func(acc: int, x: int) -> int:
                if acc is None:
                    acc = 0
                return acc + x

            return xs.pipe(_.scan(accumulator=func))

        results = scheduler.start(create).messages
        assert len(results) == 2
        assert (
            results[0].value.kind == "N"
            and results[0].time == 220
            and results[0].value.value == 2
        )
        assert results[1].value.kind == "C" and results[1].time == 250

    def test_scan_noseed_on_error(self):
        ex = "ex"
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(on_next(150, 1), on_error(250, ex))

        def create() -> Observable[int]:
            def func(acc: int, x: int) -> int:
                if acc is None:
                    acc = 0

                return acc + x

            return xs.pipe(_.scan(func))

        results = scheduler.start(create).messages
        assert len(results) == 1
        assert (
            results[0].value.kind == "E"
            and results[0].time == 250
            and str(results[0].value.exception) == ex
        )

    def test_scan_noseed_somedata(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(150, 1),
            on_next(210, 2),
            on_next(220, 3),
            on_next(230, 4),
            on_next(240, 5),
            on_completed(250),
        )

        def create():
            def func(acc, x):
                if acc is None:
                    acc = 0
                return acc + x

            return xs.pipe(_.scan(func))

        results = scheduler.start(create).messages
        assert len(results) == 5
        assert (
            results[0].value.kind == "N"
            and results[0].time == 210
            and results[0].value.value == 2
        )
        assert (
            results[1].value.kind == "N"
            and results[1].time == 220
            and results[1].value.value == 2 + 3
        )
        assert (
            results[2].value.kind == "N"
            and results[2].time == 230
            and results[2].value.value == 2 + 3 + 4
        )
        assert (
            results[3].value.kind == "N"
            and results[3].time == 240
            and results[3].value.value == 2 + 3 + 4 + 5
        )
        assert results[4].value.kind == "C" and results[4].time == 250



import unittest
from datetime import timedelta
from time import sleep

import pytest

from reactivex.internal.basic import default_now
from reactivex.scheduler.mainloop import PyGameScheduler

pygame = pytest.importorskip("pygame")


class TestPyGameScheduler(unittest.TestCase):
    def test_pygame_schedule_now(self):
        scheduler = PyGameScheduler(pygame)
        diff = scheduler.now - default_now()
        assert abs(diff) < timedelta(milliseconds=1)

    def test_pygame_schedule_now_units(self):
        scheduler = PyGameScheduler(pygame)
        diff = scheduler.now
        sleep(0.1)
        diff = scheduler.now - diff
        assert timedelta(milliseconds=80) < diff < timedelta(milliseconds=180)

    def test_pygame_schedule_action(self):
        scheduler = PyGameScheduler(pygame)
        ran = False

        def action(scheduler, state):
            nonlocal ran
            ran = True

        scheduler.schedule(action)
        scheduler.run()

        assert ran is True

    def test_pygame_schedule_action_due_relative(self):
        scheduler = PyGameScheduler(pygame)
        starttime = default_now()
        endtime = None

        def action(scheduler, state):
            nonlocal endtime
            endtime = default_now()

        scheduler.schedule_relative(0.1, action)

        scheduler.run()

        assert endtime is None

        sleep(0.2)
        scheduler.run()

        assert endtime is not None
        diff = endtime - starttime
        assert diff > timedelta(milliseconds=180)

    def test_pygame_schedule_action_due_absolute(self):
        scheduler = PyGameScheduler(pygame)
        starttime = default_now()
        endtime = None

        def action(scheduler, state):
            nonlocal endtime
            endtime = default_now()

        scheduler.schedule_absolute(starttime + timedelta(seconds=0.1), action)

        scheduler.run()

        assert endtime is None

        sleep(0.2)
        scheduler.run()

        assert endtime is not None
        diff = endtime - starttime
        assert diff > timedelta(milliseconds=180)

    def test_pygame_schedule_action_cancel(self):
        scheduler = PyGameScheduler(pygame)
        ran = False

        def action(scheduler, state):
            nonlocal ran
            ran = True

        d = scheduler.schedule_relative(0.1, action)
        d.dispose()

        sleep(0.2)
        scheduler.run()

        assert ran is False



from reactivex.subject import Subject
from reactivex.testing import ReactiveTest, TestScheduler

on_next = ReactiveTest.on_next
on_completed = ReactiveTest.on_completed
on_error = ReactiveTest.on_error
subscribe = ReactiveTest.subscribe
subscribed = ReactiveTest.subscribed
disposed = ReactiveTest.disposed
created = ReactiveTest.created


class RxException(Exception):
    pass


# Helper function for raising exceptions within lambdas
def _raise(ex):
    raise RxException(ex)


def test_infinite():
    subscription = [None]
    subscription1 = [None]
    subscription2 = [None]
    subscription3 = [None]
    s = [None]
    scheduler = TestScheduler()

    xs = scheduler.create_hot_observable(
        on_next(70, 1),
        on_next(110, 2),
        on_next(220, 3),
        on_next(270, 4),
        on_next(340, 5),
        on_next(410, 6),
        on_next(520, 7),
        on_next(630, 8),
        on_next(710, 9),
        on_next(870, 10),
        on_next(940, 11),
        on_next(1020, 12),
    )

    results1 = scheduler.create_observer()
    results2 = scheduler.create_observer()
    results3 = scheduler.create_observer()

    def action1(scheduler, state=None):
        s[0] = Subject()

    scheduler.schedule_absolute(100, action1)

    def action2(scheduler, state=None):
        subscription[0] = xs.subscribe(s[0])

    scheduler.schedule_absolute(200, action2)

    def action3(scheduler, state=None):
        subscription[0].dispose()

    scheduler.schedule_absolute(1000, action3)

    def action4(scheduler, state=None):
        subscription1[0] = s[0].subscribe(results1)

    scheduler.schedule_absolute(300, action4)

    def action5(scheduler, state=None):
        subscription2[0] = s[0].subscribe(results2)

    scheduler.schedule_absolute(400, action5)

    def action6(scheduler, state=None):
        subscription3[0] = s[0].subscribe(results3)

    scheduler.schedule_absolute(900, action6)

    def action7(scheduler, state=None):
        subscription1[0].dispose()

    scheduler.schedule_absolute(600, action7)

    def action8(scheduler, state=None):
        subscription2[0].dispose()

    scheduler.schedule_absolute(700, action8)

    def action9(scheduler, state=None):
        subscription1[0].dispose()

    scheduler.schedule_absolute(800, action9)

    def action10(scheduler, state=None):
        subscription3[0].dispose()

    scheduler.schedule_absolute(950, action10)

    scheduler.start()

    assert results1.messages == [on_next(340, 5), on_next(410, 6), on_next(520, 7)]
    assert results2.messages == [on_next(410, 6), on_next(520, 7), on_next(630, 8)]
    assert results3.messages == [on_next(940, 11)]


def test_finite():
    scheduler = TestScheduler()
    subscription = [None]
    subscription1 = [None]
    subscription2 = [None]
    subscription3 = [None]
    s = [None]

    xs = scheduler.create_hot_observable(
        on_next(70, 1),
        on_next(110, 2),
        on_next(220, 3),
        on_next(270, 4),
        on_next(340, 5),
        on_next(410, 6),
        on_next(520, 7),
        on_completed(630),
        on_next(640, 9),
        on_completed(650),
        on_error(660, "error"),
    )

    results1 = scheduler.create_observer()
    results2 = scheduler.create_observer()
    results3 = scheduler.create_observer()

    def action1(scheduler, state=None):
        s[0] = Subject()

    scheduler.schedule_absolute(100, action1)

    def action2(scheduler, state=None):
        subscription[0] = xs.subscribe(s[0])

    scheduler.schedule_absolute(200, action2)

    def action3(scheduler, state=None):
        subscription[0].dispose()

    scheduler.schedule_absolute(1000, action3)

    def action4(scheduler, state=None):
        subscription1[0] = s[0].subscribe(results1)

    scheduler.schedule_absolute(300, action4)

    def action5(scheduler, state=None):
        subscription2[0] = s[0].subscribe(results2)

    scheduler.schedule_absolute(400, action5)

    def action6(scheduler, state=None):
        subscription3[0] = s[0].subscribe(results3)

    scheduler.schedule_absolute(900, action6)

    def action7(scheduler, state=None):
        subscription1[0].dispose()

    scheduler.schedule_absolute(600, action7)

    def action8(scheduler, state=None):
        subscription2[0].dispose()

    scheduler.schedule_absolute(700, action8)

    def action9(scheduler, state=None):
        subscription1[0].dispose()

    scheduler.schedule_absolute(800, action9)

    def action10(scheduler, state=None):
        subscription3[0].dispose()

    scheduler.schedule_absolute(950, action10)

    scheduler.start()

    assert results1.messages == [on_next(340, 5), on_next(410, 6), on_next(520, 7)]
    assert results2.messages == [on_next(410, 6), on_next(520, 7), on_completed(630)]
    assert results3.messages == [on_completed(900)]


def test_error():
    s = [None]
    subscription = [None]
    subscription1 = [None]
    subscription2 = [None]
    subscription3 = [None]
    ex = "ex"

    scheduler = TestScheduler()

    xs = scheduler.create_hot_observable(
        on_next(70, 1),
        on_next(110, 2),
        on_next(220, 3),
        on_next(270, 4),
        on_next(340, 5),
        on_next(410, 6),
        on_next(520, 7),
        on_error(630, ex),
        on_next(640, 9),
        on_completed(650),
        on_error(660, "foo"),
    )

    results1 = scheduler.create_observer()
    results2 = scheduler.create_observer()
    results3 = scheduler.create_observer()

    def action(scheduler, state=None):
        s[0] = Subject()

    scheduler.schedule_absolute(100, action)

    def action1(scheduler, state=None):
        subscription[0] = xs.subscribe(s[0])

    scheduler.schedule_absolute(200, action1)

    def action2(scheduler, state=None):
        subscription[0].dispose()

    scheduler.schedule_absolute(1000, action2)

    def action3(scheduler, state=None):
        subscription1[0] = s[0].subscribe(results1)

    scheduler.schedule_absolute(300, action3)

    def action4(scheduler, state=None):
        subscription2[0] = s[0].subscribe(results2)

    scheduler.schedule_absolute(400, action4)

    def action5(scheduler, state=None):
        subscription3[0] = s[0].subscribe(results3)

    scheduler.schedule_absolute(900, action5)

    def action6(scheduler, state=None):
        subscription1[0].dispose()

    scheduler.schedule_absolute(600, action6)

    def action7(scheduler, state=None):
        subscription2[0].dispose()

    scheduler.schedule_absolute(700, action7)

    def action8(scheduler, state=None):
        subscription1[0].dispose()

    scheduler.schedule_absolute(800, action8)

    def action9(scheduler, state=None):
        subscription3[0].dispose()

    scheduler.schedule_absolute(950, action9)

    scheduler.start()

    assert results1.messages == [on_next(340, 5), on_next(410, 6), on_next(520, 7)]
    assert results2.messages == [on_next(410, 6), on_next(520, 7), on_error(630, ex)]
    assert results3.messages == [on_error(900, ex)]


def test_canceled():
    s = [None]
    subscription = [None]
    subscription1 = [None]
    subscription2 = [None]
    subscription3 = [None]

    scheduler = TestScheduler()
    xs = scheduler.create_hot_observable(
        on_completed(630), on_next(640, 9), on_completed(650), on_error(660, "ex")
    )

    results1 = scheduler.create_observer()
    results2 = scheduler.create_observer()
    results3 = scheduler.create_observer()

    def action1(scheduler, state=None):
        s[0] = Subject()

    scheduler.schedule_absolute(100, action1)

    def action2(scheduler, state=None):
        subscription[0] = xs.subscribe(s[0])

    scheduler.schedule_absolute(200, action2)

    def action3(scheduler, state=None):
        subscription[0].dispose()

    scheduler.schedule_absolute(1000, action3)

    def action4(scheduler, state=None):
        subscription1[0] = s[0].subscribe(results1)

    scheduler.schedule_absolute(300, action4)

    def action5(scheduler, state=None):
        subscription2[0] = s[0].subscribe(results2)

    scheduler.schedule_absolute(400, action5)

    def action6(scheduler, state=None):
        subscription3[0] = s[0].subscribe(results3)

    scheduler.schedule_absolute(900, action6)

    def action7(scheduler, state=None):
        subscription1[0].dispose()

    scheduler.schedule_absolute(600, action7)

    def action8(scheduler, state=None):
        subscription2[0].dispose()

    scheduler.schedule_absolute(700, action8)

    def action9(scheduler, state=None):
        subscription1[0].dispose()

    scheduler.schedule_absolute(800, action9)

    def action10(scheduler, state=None):
        subscription3[0].dispose()

    scheduler.schedule_absolute(950, action10)

    scheduler.start()

    assert results1.messages == []
    assert results2.messages == [on_completed(630)]
    assert results3.messages == [on_completed(900)]



import unittest

import reactivex
from reactivex import operators as ops
from reactivex.testing import ReactiveTest, TestScheduler

on_next = ReactiveTest.on_next
on_completed = ReactiveTest.on_completed
on_error = ReactiveTest.on_error
subscribe = ReactiveTest.subscribe
subscribed = ReactiveTest.subscribed
disposed = ReactiveTest.disposed
created = ReactiveTest.created


class RxException(Exception):
    pass


# Helper function for raising exceptions within lambdas
def _raise(ex):
    raise RxException(ex)


class TestDistinctUntilChanged(unittest.TestCase):
    def test_distinct_until_changed_never(self):
        scheduler = TestScheduler()

        def create():
            return reactivex.never().pipe(ops.distinct_until_changed())

        results = scheduler.start(create)

        assert results.messages == []

    def test_distinct_until_changed_empty(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(on_next(150, 1), on_completed(250))

        def create():
            return xs.pipe(ops.distinct_until_changed())

        results = scheduler.start(create).messages
        self.assertEqual(1, len(results))
        assert results[0].value.kind == "C" and results[0].time == 250

    def test_distinct_until_changed_return(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(150, 1), on_next(220, 2), on_completed(250)
        )

        def create():
            return xs.pipe(ops.distinct_until_changed())

        results = scheduler.start(create).messages
        self.assertEqual(2, len(results))
        assert (
            results[0].value.kind == "N"
            and results[0].time == 220
            and results[0].value.value == 2
        )
        assert results[1].value.kind == "C" and results[1].time == 250

    def test_distinct_until_changed_on_error(self):
        ex = "ex"
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(on_next(150, 1), on_error(250, ex))

        def create():
            return xs.pipe(ops.distinct_until_changed())

        results = scheduler.start(create).messages
        self.assertEqual(1, len(results))
        assert (
            results[0].value.kind == "E"
            and results[0].time == 250
            and str(results[0].value.exception) == ex
        )

    def test_distinct_until_changed_all_changes(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(150, 1),
            on_next(210, 2),
            on_next(220, 3),
            on_next(230, 4),
            on_next(240, 5),
            on_completed(250),
        )

        def create():
            return xs.pipe(ops.distinct_until_changed())

        results = scheduler.start(create).messages
        self.assertEqual(5, len(results))
        assert (
            results[0].value.kind == "N"
            and results[0].time == 210
            and results[0].value.value == 2
        )
        assert (
            results[1].value.kind == "N"
            and results[1].time == 220
            and results[1].value.value == 3
        )
        assert (
            results[2].value.kind == "N"
            and results[2].time == 230
            and results[2].value.value == 4
        )
        assert (
            results[3].value.kind == "N"
            and results[3].time == 240
            and results[3].value.value == 5
        )
        assert results[4].value.kind == "C" and results[4].time == 250

    def test_distinct_until_changed_all_same(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(150, 1),
            on_next(210, 2),
            on_next(220, 2),
            on_next(230, 2),
            on_next(240, 2),
            on_completed(250),
        )

        def create():
            return xs.pipe(ops.distinct_until_changed())

        results = scheduler.start(create).messages
        self.assertEqual(2, len(results))
        assert (
            results[0].value.kind == "N"
            and results[0].time == 210
            and results[0].value.value == 2
        )
        assert results[1].value.kind == "C" and results[1].time == 250

    def test_distinct_until_changed_some_changes(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(150, 1),
            on_next(210, 2),
            on_next(215, 3),
            on_next(220, 3),
            on_next(225, 2),
            on_next(230, 2),
            on_next(230, 1),
            on_next(240, 2),
            on_completed(250),
        )

        def create():
            return xs.pipe(ops.distinct_until_changed())

        results = scheduler.start(create).messages
        self.assertEqual(6, len(results))
        assert (
            results[0].value.kind == "N"
            and results[0].time == 210
            and results[0].value.value == 2
        )
        assert (
            results[1].value.kind == "N"
            and results[1].time == 215
            and results[1].value.value == 3
        )
        assert (
            results[2].value.kind == "N"
            and results[2].time == 225
            and results[2].value.value == 2
        )
        assert (
            results[3].value.kind == "N"
            and results[3].time == 230
            and results[3].value.value == 1
        )
        assert (
            results[4].value.kind == "N"
            and results[4].time == 240
            and results[4].value.value == 2
        )
        assert results[5].value.kind == "C" and results[5].time == 250

    def test_distinct_until_changed_comparer_all_equal(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(150, 1),
            on_next(210, 2),
            on_next(220, 3),
            on_next(230, 4),
            on_next(240, 5),
            on_completed(250),
        )

        def create():
            return xs.pipe(ops.distinct_until_changed(comparer=lambda x, y: True))

        results = scheduler.start(create).messages
        self.assertEqual(2, len(results))
        assert (
            results[0].value.kind == "N"
            and results[0].time == 210
            and results[0].value.value == 2
        )
        assert results[1].value.kind == "C" and results[1].time == 250

    def test_distinct_until_changed_comparer_all_different(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(150, 1),
            on_next(210, 2),
            on_next(220, 2),
            on_next(230, 2),
            on_next(240, 2),
            on_completed(250),
        )

        def create():
            return xs.pipe(ops.distinct_until_changed(comparer=lambda x, y: False))

        results = scheduler.start(create).messages
        self.assertEqual(5, len(results))
        assert (
            results[0].value.kind == "N"
            and results[0].time == 210
            and results[0].value.value == 2
        )
        assert (
            results[1].value.kind == "N"
            and results[1].time == 220
            and results[1].value.value == 2
        )
        assert (
            results[2].value.kind == "N"
            and results[2].time == 230
            and results[2].value.value == 2
        )
        assert (
            results[3].value.kind == "N"
            and results[3].time == 240
            and results[3].value.value == 2
        )
        assert results[4].value.kind == "C" and results[4].time == 250

    def test_distinct_until_changed_key_mapper_div2(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(150, 1),
            on_next(210, 2),
            on_next(220, 4),
            on_next(230, 3),
            on_next(240, 5),
            on_completed(250),
        )

        def create():
            return xs.pipe(ops.distinct_until_changed(lambda x: x % 2))

        results = scheduler.start(create).messages
        self.assertEqual(3, len(results))
        assert (
            results[0].value.kind == "N"
            and results[0].time == 210
            and results[0].value.value == 2
        )
        assert (
            results[1].value.kind == "N"
            and results[1].time == 230
            and results[1].value.value == 3
        )
        assert results[2].value.kind == "C" and results[2].time == 250

    def test_distinct_until_changed_key_mapper_throws(self):
        ex = "ex"
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(150, 1), on_next(210, 2), on_completed(250)
        )

        def create():
            return xs.pipe(ops.distinct_until_changed(lambda x: _raise(ex)))

        results = scheduler.start(create)
        assert results.messages == [on_error(210, ex)]

    def test_distinct_until_changed_comparer_throws(self):
        ex = "ex"
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(150, 1), on_next(210, 2), on_next(220, 3), on_completed(250)
        )

        def create():
            return xs.pipe(
                ops.distinct_until_changed(comparer=lambda x, y: _raise(ex)),
            )

        results = scheduler.start(create)
        assert results.messages == [on_next(210, 2), on_error(220, ex)]



import asyncio
import unittest

import reactivex
import reactivex.operators as ops
from reactivex.internal.exceptions import SequenceContainsNoElementsError
from reactivex.subject import Subject
from reactivex.testing import ReactiveTest

on_next = ReactiveTest.on_next
on_completed = ReactiveTest.on_completed
on_error = ReactiveTest.on_error
subscribe = ReactiveTest.subscribe
subscribed = ReactiveTest.subscribed
disposed = ReactiveTest.disposed
created = ReactiveTest.created


class TestToFuture(unittest.TestCase):
    def test_await_success(self):
        loop = asyncio.get_event_loop()
        result = None

        async def go():
            nonlocal result
            source = reactivex.return_value(42)
            result = await source

        loop.run_until_complete(go())
        assert result == 42

    def test_await_success_on_sequence(self):
        loop = asyncio.get_event_loop()
        result = None

        async def go():
            nonlocal result
            source = reactivex.from_([40, 41, 42])
            result = await source

        loop.run_until_complete(go())
        assert result == 42

    def test_await_error(self):
        loop = asyncio.get_event_loop()
        error = Exception("error")
        result = None

        async def go():
            nonlocal result
            source = reactivex.throw(error)
            try:
                result = await source
            except Exception as ex:
                result = ex

        loop.run_until_complete(go())
        assert result == error

    def test_await_empty_observable(self):
        loop = asyncio.get_event_loop()
        result = None

        async def go():
            nonlocal result
            source = reactivex.empty()
            result = await source

        self.assertRaises(
            SequenceContainsNoElementsError, loop.run_until_complete, go()
        )

    def test_await_with_delay(self):
        loop = asyncio.get_event_loop()
        result = None

        async def go():
            nonlocal result
            source = reactivex.return_value(42).pipe(ops.delay(0.1))
            result = await source

        loop.run_until_complete(go())
        assert result == 42

    def test_cancel(self):
        loop = asyncio.get_event_loop()

        async def go():
            source = reactivex.return_value(42)
            fut = next(source.__await__())
            # This used to raise an InvalidStateError before we got
            # support for cancellation.
            fut.cancel()
            await fut

        self.assertRaises(asyncio.CancelledError, loop.run_until_complete, go())

    def test_dispose_on_cancel(self):
        loop = asyncio.get_event_loop()
        sub = Subject()

        async def using_sub():
            # Since the subject never completes, this await statement
            # will never be complete either. We wait forever.
            await reactivex.using(lambda: sub, lambda s: s)

        async def go():
            await asyncio.wait_for(using_sub(), 0.1)

        self.assertRaises(asyncio.TimeoutError, loop.run_until_complete, go())
        # When we cancel the future (due to the time-out), the future
        # automatically disposes the underlying subject.
        self.assertTrue(sub.is_disposed)



import unittest

import reactivex
from reactivex import ConnectableObservable, Observable
from reactivex import operators as ops
from reactivex.abc import ObserverBase
from reactivex.testing import ReactiveTest, TestScheduler

on_next = ReactiveTest.on_next
on_completed = ReactiveTest.on_completed
on_error = ReactiveTest.on_error
subscribe = ReactiveTest.subscribe
subscribed = ReactiveTest.subscribed
disposed = ReactiveTest.disposed
created = ReactiveTest.created


class RxException(Exception):
    pass


# Helper function for raising exceptions within lambdas
def _raise(ex):
    raise RxException(ex)


class MySubject(Observable, ObserverBase):
    def __init__(self):
        super(MySubject, self).__init__()

        self.dispose_on_map = {}
        self.subscribe_count = 0
        self.disposed = False
        self.observer = None

    def _subscribe_core(self, observer, scheduler=None):
        self.subscribe_count += 1
        self.observer = observer

        class Duck:
            def __init__(self, this):
                self.this = this

            def dispose(self):
                self.this.disposed = True

        return Duck(self)

    def dispose_on(self, value, disposable):
        self.dispose_on_map[value] = disposable

    def on_next(self, value):
        self.observer.on_next(value)
        if value in self.dispose_on_map:
            self.dispose_on_map[value].dispose()

    def on_error(self, error):
        self.observer.on_error(error)

    def on_completed(self):
        self.observer.on_completed()


class TestPublish(unittest.TestCase):
    def test_publish_cold_zip(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(40, 0),
            on_next(90, 1),
            on_next(150, 2),
            on_next(210, 3),
            on_next(240, 4),
            on_next(270, 5),
            on_next(330, 6),
            on_next(340, 7),
            on_completed(390),
        )

        def create():
            def mapper(ys):
                return ys.pipe(
                    ops.zip(ys),
                    ops.map(sum),
                )

            return xs.pipe(ops.publish(mapper=mapper))

        results = scheduler.start(create)

        assert results.messages == [
            on_next(210, 6),
            on_next(240, 8),
            on_next(270, 10),
            on_next(330, 12),
            on_next(340, 14),
            on_completed(390),
        ]
        assert xs.subscriptions == [subscribe(200, 390)]

    def test_ref_count_connects_on_first(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(210, 1),
            on_next(220, 2),
            on_next(230, 3),
            on_next(240, 4),
            on_completed(250),
        )
        subject = MySubject()
        conn = ConnectableObservable(xs, subject)

        def create():
            return conn.pipe(ops.ref_count())

        res = scheduler.start(create)

        assert res.messages == [
            on_next(210, 1),
            on_next(220, 2),
            on_next(230, 3),
            on_next(240, 4),
            on_completed(250),
        ]
        assert subject.disposed

    def test_ref_count_notconnected(self):
        disconnected = [False]
        count = [0]

        def factory(scheduler):
            count[0] += 1

            def create(obs, scheduler=None):
                def func():
                    disconnected[0] = True

                return func

            return reactivex.create(create)

        xs = reactivex.defer(factory)

        subject = MySubject()
        conn = ConnectableObservable(xs, subject)
        refd = conn.pipe(ops.ref_count())
        dis1 = refd.subscribe()
        self.assertEqual(1, count[0])
        self.assertEqual(1, subject.subscribe_count)
        assert not disconnected[0]
        dis2 = refd.subscribe()
        self.assertEqual(1, count[0])
        self.assertEqual(2, subject.subscribe_count)
        assert not disconnected[0]
        dis1.dispose()
        assert not disconnected[0]
        dis2.dispose()
        assert disconnected[0]
        disconnected[0] = False
        dis3 = refd.subscribe()
        self.assertEqual(2, count[0])
        self.assertEqual(3, subject.subscribe_count)
        assert not disconnected[0]
        dis3.dispose()
        assert disconnected[0]

    def test_publish_basic(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(110, 7),
            on_next(220, 3),
            on_next(280, 4),
            on_next(290, 1),
            on_next(340, 8),
            on_next(360, 5),
            on_next(370, 6),
            on_next(390, 7),
            on_next(410, 13),
            on_next(430, 2),
            on_next(450, 9),
            on_next(520, 11),
            on_next(560, 20),
            on_completed(600),
        )
        ys = [None]
        subscription = [None]
        connection = [None]
        results = scheduler.create_observer()

        def action0(scheduler, state):
            ys[0] = xs.pipe(ops.publish())

        scheduler.schedule_absolute(created, action0)

        def action1(scheduler, state):
            subscription[0] = ys[0].subscribe(results)

        scheduler.schedule_absolute(subscribed, action1)

        def action2(scheduler, state):
            subscription[0].dispose()

        scheduler.schedule_absolute(disposed, action2)

        def action3(scheduler, state):
            connection[0] = ys[0].connect()

        scheduler.schedule_absolute(300, action3)

        def action4(scheduler, state):
            connection[0].dispose()

        scheduler.schedule_absolute(400, action4)

        def action5(scheduler, state):
            connection[0] = ys[0].connect()

        scheduler.schedule_absolute(500, action5)

        def action6(scheduler, state):
            connection[0].dispose()

        scheduler.schedule_absolute(550, action6)

        def action7(scheduler, state):
            connection[0] = ys[0].connect()

        scheduler.schedule_absolute(650, action7)

        def action8(scheduler, state):
            connection[0].dispose()

        scheduler.schedule_absolute(800, action8)

        scheduler.start()
        assert results.messages == [
            on_next(340, 8),
            on_next(360, 5),
            on_next(370, 6),
            on_next(390, 7),
            on_next(520, 11),
        ]
        assert xs.subscriptions == [
            subscribe(300, 400),
            subscribe(500, 550),
            subscribe(650, 800),
        ]

    def test_publish_error(self):
        ex = "ex"
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(110, 7),
            on_next(220, 3),
            on_next(280, 4),
            on_next(290, 1),
            on_next(340, 8),
            on_next(360, 5),
            on_next(370, 6),
            on_next(390, 7),
            on_next(410, 13),
            on_next(430, 2),
            on_next(450, 9),
            on_next(520, 11),
            on_next(560, 20),
            on_error(600, ex),
        )

        ys = [None]
        subscription = [None]
        connection = [None]
        results = scheduler.create_observer()

        def action0(scheduler, state):
            ys[0] = xs.pipe(ops.publish())

        scheduler.schedule_absolute(created, action0)

        def action1(scheduler, state):
            subscription[0] = ys[0].subscribe(results)

        scheduler.schedule_absolute(subscribed, action1)

        def action2(scheduler, state):
            subscription[0].dispose()

        scheduler.schedule_absolute(disposed, action2)

        def action3(scheduler, state):
            connection[0] = ys[0].connect()

        scheduler.schedule_absolute(300, action3)

        def action4(scheduler, state):
            connection[0].dispose()

        scheduler.schedule_absolute(400, action4)

        def action5(scheduler, state):
            connection[0] = ys[0].connect()

        scheduler.schedule_absolute(500, action5)

        def action6(scheduler, state):
            connection[0].dispose()

        scheduler.schedule_absolute(800, action6)

        scheduler.start()
        assert results.messages == [
            on_next(340, 8),
            on_next(360, 5),
            on_next(370, 6),
            on_next(390, 7),
            on_next(520, 11),
            on_next(560, 20),
            on_error(600, ex),
        ]
        assert xs.subscriptions == [subscribe(300, 400), subscribe(500, 600)]

    def test_publish_complete(self):
        connection = [None]
        subscription = [None]
        ys = [None]

        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(110, 7),
            on_next(220, 3),
            on_next(280, 4),
            on_next(290, 1),
            on_next(340, 8),
            on_next(360, 5),
            on_next(370, 6),
            on_next(390, 7),
            on_next(410, 13),
            on_next(430, 2),
            on_next(450, 9),
            on_next(520, 11),
            on_next(560, 20),
            on_completed(600),
        )
        results = scheduler.create_observer()

        def action0(scheduler, state):
            ys[0] = xs.pipe(ops.publish())

        scheduler.schedule_absolute(created, action0)

        def action1(scheduler, state):
            subscription[0] = ys[0].subscribe(results)

        scheduler.schedule_absolute(subscribed, action1)

        def action2(scheduler, state):
            subscription[0].dispose()

        scheduler.schedule_absolute(disposed, action2)

        def action3(scheduler, state):
            connection[0] = ys[0].connect()

        scheduler.schedule_absolute(300, action3)

        def action4(scheduler, state):
            connection[0].dispose()

        scheduler.schedule_absolute(400, action4)

        def action5(scheduler, state):
            connection[0] = ys[0].connect()

        scheduler.schedule_absolute(500, action5)

        def action6(scheduler, state):
            connection[0].dispose()

        scheduler.schedule_absolute(800, action6)

        scheduler.start()
        assert results.messages == [
            on_next(340, 8),
            on_next(360, 5),
            on_next(370, 6),
            on_next(390, 7),
            on_next(520, 11),
            on_next(560, 20),
            on_completed(600),
        ]
        assert xs.subscriptions == [subscribe(300, 400), subscribe(500, 600)]

    def test_publish_dispose(self):
        connection = [None]
        subscription = [None]
        ys = [None]
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(110, 7),
            on_next(220, 3),
            on_next(280, 4),
            on_next(290, 1),
            on_next(340, 8),
            on_next(360, 5),
            on_next(370, 6),
            on_next(390, 7),
            on_next(410, 13),
            on_next(430, 2),
            on_next(450, 9),
            on_next(520, 11),
            on_next(560, 20),
            on_completed(600),
        )

        results = scheduler.create_observer()

        def action0(scheduler, state):
            ys[0] = xs.pipe(ops.publish())

        scheduler.schedule_absolute(created, action0)

        def action1(scheduler, state):
            subscription[0] = ys[0].subscribe(results)

        scheduler.schedule_absolute(subscribed, action1)

        def action2(scheduler, state):
            subscription[0].dispose()

        scheduler.schedule_absolute(350, action2)

        def action3(scheduler, state):
            connection[0] = ys[0].connect()

        scheduler.schedule_absolute(300, action3)

        def action4(scheduler, state):
            connection[0].dispose()

        scheduler.schedule_absolute(400, action4)

        def action5(scheduler, state):
            connection[0] = ys[0].connect()

        scheduler.schedule_absolute(500, action5)

        def action6(scheduler, state):
            connection[0].dispose()

        scheduler.schedule_absolute(550, action6)

        def action7(scheduler, state):
            connection[0] = ys[0].connect()

        scheduler.schedule_absolute(650, action7)

        def action8(scheduler, state):
            connection[0].dispose()

        scheduler.schedule_absolute(800, action8)

        scheduler.start()
        assert results.messages == [on_next(340, 8)]
        assert xs.subscriptions == [
            subscribe(300, 400),
            subscribe(500, 550),
            subscribe(650, 800),
        ]

    def test_publish_multipleconnections(self):
        xs = reactivex.never()
        ys = xs.pipe(ops.publish())
        connection1 = ys.connect()
        connection2 = ys.connect()
        assert connection1 == connection2
        connection1.dispose()
        connection2.dispose()
        connection3 = ys.connect()
        assert connection1 != connection3
        connection3.dispose()

    def test_publish_lambda_zip_complete(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(110, 7),
            on_next(220, 3),
            on_next(280, 4),
            on_next(290, 1),
            on_next(340, 8),
            on_next(360, 5),
            on_next(370, 6),
            on_next(390, 7),
            on_next(410, 13),
            on_next(430, 2),
            on_next(450, 9),
            on_next(520, 11),
            on_next(560, 20),
            on_completed(600),
        )

        def create():
            def mapper(_xs):
                return _xs.pipe(
                    ops.zip(_xs.pipe(ops.skip(1))),
                    ops.map(sum),
                )

            return xs.pipe(ops.publish(mapper))

        results = scheduler.start(create)

        assert results.messages == [
            on_next(280, 7),
            on_next(290, 5),
            on_next(340, 9),
            on_next(360, 13),
            on_next(370, 11),
            on_next(390, 13),
            on_next(410, 20),
            on_next(430, 15),
            on_next(450, 11),
            on_next(520, 20),
            on_next(560, 31),
            on_completed(600),
        ]
        assert xs.subscriptions == [subscribe(200, 600)]

    def test_publish_lambda_zip_error(self):
        ex = "ex"
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(110, 7),
            on_next(220, 3),
            on_next(280, 4),
            on_next(290, 1),
            on_next(340, 8),
            on_next(360, 5),
            on_next(370, 6),
            on_next(390, 7),
            on_next(410, 13),
            on_next(430, 2),
            on_next(450, 9),
            on_next(520, 11),
            on_next(560, 20),
            on_error(600, ex),
        )

        def create():
            def mapper(_xs):
                return _xs.pipe(
                    ops.zip(_xs.pipe(ops.skip(1))),
                    ops.map(sum),
                )

            return xs.pipe(ops.publish(mapper))

        results = scheduler.start(create)

        assert results.messages == [
            on_next(280, 7),
            on_next(290, 5),
            on_next(340, 9),
            on_next(360, 13),
            on_next(370, 11),
            on_next(390, 13),
            on_next(410, 20),
            on_next(430, 15),
            on_next(450, 11),
            on_next(520, 20),
            on_next(560, 31),
            on_error(600, ex),
        ]
        assert xs.subscriptions == [subscribe(200, 600)]

    def test_publish_lambda_zip_dispose(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(110, 7),
            on_next(220, 3),
            on_next(280, 4),
            on_next(290, 1),
            on_next(340, 8),
            on_next(360, 5),
            on_next(370, 6),
            on_next(390, 7),
            on_next(410, 13),
            on_next(430, 2),
            on_next(450, 9),
            on_next(520, 11),
            on_next(560, 20),
            on_completed(600),
        )

        def create():
            def mapper(_xs):
                return _xs.pipe(ops.zip(_xs.pipe(ops.skip(1))), ops.map(sum))

            return xs.pipe(ops.publish(mapper))

        results = scheduler.start(create, disposed=470)
        assert results.messages == [
            on_next(280, 7),
            on_next(290, 5),
            on_next(340, 9),
            on_next(360, 13),
            on_next(370, 11),
            on_next(390, 13),
            on_next(410, 20),
            on_next(430, 15),
            on_next(450, 11),
        ]
        assert xs.subscriptions == [subscribe(200, 470)]



import unittest

from reactivex import operators as ops
from reactivex.testing import ReactiveTest, TestScheduler

on_next = ReactiveTest.on_next
on_completed = ReactiveTest.on_completed
on_error = ReactiveTest.on_error
subscribe = ReactiveTest.subscribe
subscribed = ReactiveTest.subscribed
disposed = ReactiveTest.disposed
created = ReactiveTest.created


class TestIgnoreElements(unittest.TestCase):
    def test_ignore_values_basic(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(180, 1),
            on_next(210, 2),
            on_next(250, 3),
            on_next(270, 4),
            on_next(310, 5),
            on_next(360, 6),
            on_next(380, 7),
            on_next(410, 8),
            on_next(590, 9),
        )
        results = scheduler.start(create=lambda: xs.pipe(ops.ignore_elements()))

        assert results.messages == []
        assert xs.subscriptions == [subscribe(200, 1000)]

    def test_ignore_values_completed(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(180, 1),
            on_next(210, 2),
            on_next(250, 3),
            on_next(270, 4),
            on_next(310, 5),
            on_next(360, 6),
            on_next(380, 7),
            on_next(410, 8),
            on_next(590, 9),
            on_completed(610),
        )
        results = scheduler.start(create=lambda: xs.pipe(ops.ignore_elements()))

        assert results.messages == [on_completed(610)]
        assert xs.subscriptions == [subscribe(200, 610)]

    def test_ignore_values_error(self):
        ex = "ex"
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(180, 1),
            on_next(210, 2),
            on_next(250, 3),
            on_next(270, 4),
            on_next(310, 5),
            on_next(360, 6),
            on_next(380, 7),
            on_next(410, 8),
            on_next(590, 9),
            on_error(610, ex),
        )
        results = scheduler.start(create=lambda: xs.pipe(ops.ignore_elements()))

        assert results.messages == [on_error(610, ex)]
        assert xs.subscriptions == [subscribe(200, 610)]



import unittest
from datetime import datetime, timedelta, timezone

from reactivex.internal.constants import UTC_ZERO
from reactivex.scheduler import HistoricalScheduler


def assert_equals(first, second):
    if len(first) != len(second):
        print("len(%d) != len(%d)" % (len(first), len(second)))
        assert False

    for i in range(len(first)):
        f = first[i]
        s = second[i]
        if hasattr(f, "assert_equals") and hasattr(s, "assert_equals"):
            assert f.assert_equals(s)
        else:
            assert f == s


def time(days):
    dt = datetime(
        year=1979, month=10, day=31, hour=4, minute=30, second=15, tzinfo=timezone.utc
    )
    dt = dt + timedelta(days=days)
    return dt


def from_days(days):
    return timedelta(days=days)


class Timestamped(object):
    def __init__(self, value, timestamp):
        self.value = value
        self.timestamp = timestamp

    def assert_equals(self, other):
        if not other:
            return False

        return other.value == self.value and other.timestamp == self.timestamp

    def __str__(self):
        return "(%s, %s)" % (self.value, self.timestamp)

    def __repr__(self):
        return str(self)


class TestHistoricalScheduler(unittest.TestCase):
    def test_ctor(self):
        s = HistoricalScheduler()
        self.assertEqual(UTC_ZERO, s.clock)
        self.assertEqual(False, s._is_enabled)

    def test_start_stop(self):
        s = HistoricalScheduler()
        list = []

        s.schedule_absolute(time(0), lambda sc, st: list.append(Timestamped(1, s.now)))
        s.schedule_absolute(time(1), lambda sc, st: list.append(Timestamped(2, s.now)))
        s.schedule_absolute(time(2), lambda sc, st: s.stop())
        s.schedule_absolute(time(3), lambda sc, st: list.append(Timestamped(3, s.now)))
        s.schedule_absolute(time(4), lambda sc, st: s.stop())
        s.schedule_absolute(time(5), lambda sc, st: s.start())
        s.schedule_absolute(time(6), lambda sc, st: list.append(Timestamped(4, s.now)))

        s.start()

        self.assertEqual(time(2), s.now)
        self.assertEqual(time(2), s.clock)

        s.start()

        self.assertEqual(time(4), s.now)
        self.assertEqual(time(4), s.clock)

        s.start()

        self.assertEqual(time(6), s.now)
        self.assertEqual(time(6), s.clock)

        s.start()

        self.assertEqual(time(6), s.now)
        self.assertEqual(time(6), s.clock)

        assert_equals(
            list,
            [
                Timestamped(1, time(0)),
                Timestamped(2, time(1)),
                Timestamped(3, time(3)),
                Timestamped(4, time(6)),
            ],
        )

    def test_order(self):
        s = HistoricalScheduler()

        list = []

        s.schedule_absolute(time(2), lambda a, b: list.append(Timestamped(2, s.now)))

        s.schedule_absolute(time(3), lambda a, b: list.append(Timestamped(3, s.now)))

        s.schedule_absolute(time(1), lambda a, b: list.append(Timestamped(0, s.now)))
        s.schedule_absolute(time(1), lambda a, b: list.append(Timestamped(1, s.now)))

        s.start()

        assert_equals(
            list,
            [
                Timestamped(0, time(1)),
                Timestamped(1, time(1)),
                Timestamped(2, time(2)),
                Timestamped(3, time(3)),
            ],
        )

    def test_cancellation(self):
        s = HistoricalScheduler()

        list = []

        d = s.schedule_absolute(
            time(2), lambda a, b: list.append(Timestamped(2, s.now))
        )

        def action(scheduler, state):
            list.append(Timestamped(0, s.now))
            d.dispose()

        s.schedule_absolute(time(1), action)

        s.start()

        assert_equals(list, [Timestamped(0, time(1))])

    def test_advance_to(self):
        s = HistoricalScheduler()

        list = []

        s.schedule_absolute(time(0), lambda a, b: list.append(Timestamped(0, s.now)))
        s.schedule_absolute(time(1), lambda a, b: list.append(Timestamped(1, s.now)))
        s.schedule_absolute(time(2), lambda a, b: list.append(Timestamped(2, s.now)))
        s.schedule_absolute(time(10), lambda a, b: list.append(Timestamped(10, s.now)))
        s.schedule_absolute(time(11), lambda a, b: list.append(Timestamped(11, s.now)))

        s.advance_to(time(8))

        self.assertEqual(time(8), s.now)
        self.assertEqual(time(8), s.clock)

        assert_equals(
            list,
            [Timestamped(0, time(0)), Timestamped(1, time(1)), Timestamped(2, time(2))],
        )

        s.advance_to(time(8))

        self.assertEqual(time(8), s.now)
        self.assertEqual(time(8), s.clock)

        assert_equals(
            list,
            [Timestamped(0, time(0)), Timestamped(1, time(1)), Timestamped(2, time(2))],
        )

        s.schedule_absolute(time(7), lambda a, b: list.append(Timestamped(7, s.now)))
        s.schedule_absolute(time(8), lambda a, b: list.append(Timestamped(8, s.now)))

        self.assertEqual(time(8), s.now)
        self.assertEqual(time(8), s.clock)

        assert_equals(
            list,
            [Timestamped(0, time(0)), Timestamped(1, time(1)), Timestamped(2, time(2))],
        )

        s.advance_to(time(10))

        self.assertEqual(time(10), s.now)
        self.assertEqual(time(10), s.clock)

        assert_equals(
            list,
            [
                Timestamped(0, time(0)),
                Timestamped(1, time(1)),
                Timestamped(2, time(2)),
                Timestamped(7, time(8)),
                Timestamped(8, time(8)),
                Timestamped(10, time(10)),
            ],
        )

        s.advance_to(time(100))

        self.assertEqual(time(100), s.now)
        self.assertEqual(time(100), s.clock)

        assert_equals(
            list,
            [
                Timestamped(0, time(0)),
                Timestamped(1, time(1)),
                Timestamped(2, time(2)),
                Timestamped(7, time(8)),
                Timestamped(8, time(8)),
                Timestamped(10, time(10)),
                Timestamped(11, time(11)),
            ],
        )

    def test_advance_by(self):
        s = HistoricalScheduler()

        list = []

        s.schedule_absolute(time(0), lambda a, b: list.append(Timestamped(0, s.now)))
        s.schedule_absolute(time(1), lambda a, b: list.append(Timestamped(1, s.now)))
        s.schedule_absolute(time(2), lambda a, b: list.append(Timestamped(2, s.now)))
        s.schedule_absolute(time(10), lambda a, b: list.append(Timestamped(10, s.now)))
        s.schedule_absolute(time(11), lambda a, b: list.append(Timestamped(11, s.now)))

        s.advance_by(time(8) - s.now)

        self.assertEqual(time(8), s.now)
        self.assertEqual(time(8), s.clock)

        assert_equals(
            list,
            [Timestamped(0, time(0)), Timestamped(1, time(1)), Timestamped(2, time(2))],
        )

        s.schedule_absolute(time(7), lambda a, b: list.append(Timestamped(7, s.now)))
        s.schedule_absolute(time(8), lambda a, b: list.append(Timestamped(8, s.now)))

        self.assertEqual(time(8), s.now)
        self.assertEqual(time(8), s.clock)

        assert_equals(
            list,
            [Timestamped(0, time(0)), Timestamped(1, time(1)), Timestamped(2, time(2))],
        )

        s.advance_by(timedelta(0))

        self.assertEqual(time(8), s.now)
        self.assertEqual(time(8), s.clock)

        assert_equals(
            list,
            [Timestamped(0, time(0)), Timestamped(1, time(1)), Timestamped(2, time(2))],
        )

        s.advance_by(from_days(2))

        self.assertEqual(time(10), s.now)
        self.assertEqual(time(10), s.clock)

        assert_equals(
            list,
            [
                Timestamped(0, time(0)),
                Timestamped(1, time(1)),
                Timestamped(2, time(2)),
                Timestamped(7, time(8)),
                Timestamped(8, time(8)),
                Timestamped(10, time(10)),
            ],
        )

        s.advance_by(from_days(90))

        self.assertEqual(time(100), s.now)
        self.assertEqual(time(100), s.clock)

        assert_equals(
            list,
            [
                Timestamped(0, time(0)),
                Timestamped(1, time(1)),
                Timestamped(2, time(2)),
                Timestamped(7, time(8)),
                Timestamped(8, time(8)),
                Timestamped(10, time(10)),
                Timestamped(11, time(11)),
            ],
        )

    def test_is_enabled(self):
        s = HistoricalScheduler()

        self.assertEqual(False, s._is_enabled)

        def action(scheduler, state):
            self.assertEqual(True, s._is_enabled)
            s.stop()
            self.assertEqual(False, s._is_enabled)

        s.schedule(action)

        self.assertEqual(False, s._is_enabled)

        s.start()

        self.assertEqual(False, s._is_enabled)

    def test_sleep1(self):
        now = datetime(year=1983, month=2, day=11, hour=12)

        s = HistoricalScheduler(now)

        s.sleep(from_days(1))

        self.assertEqual(now + from_days(1), s.clock)

    def test_sleep2(self):
        s = HistoricalScheduler()
        n = [0]

        def action(scheduler, state):
            s.sleep(timedelta(3 * 6000))
            n[0] += 1
            s.schedule_absolute(s.now + timedelta(6000), action)

        s.schedule_absolute(s.now + timedelta(6000), action)

        s.advance_to(s.now + timedelta(5 * 6000))

        self.assertEqual(2, n[0])

    def test_schedule_relative_with_timedelta(self):
        s = HistoricalScheduler()
        n = 0

        def action(scheduler, state):
            nonlocal n
            n += 1

        s.schedule_relative(timedelta(2), action)

        s.advance_by(timedelta(1))

        self.assertEqual(n, 0)

        s.advance_by(timedelta(1))

        self.assertEqual(n, 1)

    def test_schedule_relative_with_float(self):
        s = HistoricalScheduler()
        n = 0

        def action(scheduler, state):
            nonlocal n
            n += 1

        s.schedule_relative(1.0, action)

        s.advance_by(0.5)

        self.assertEqual(n, 0)

        s.advance_by(0.5)

        self.assertEqual(n, 1)



import unittest

from reactivex import operators as ops
from reactivex.testing import ReactiveTest, TestScheduler, is_prime

on_next = ReactiveTest.on_next
on_completed = ReactiveTest.on_completed
on_error = ReactiveTest.on_error
subscribe = ReactiveTest.subscribe
subscribed = ReactiveTest.subscribed
disposed = ReactiveTest.disposed
created = ReactiveTest.created


class TestTakeWhile(unittest.TestCase):
    def test_take_while_complete_Before(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(90, -1),
            on_next(110, -1),
            on_next(210, 2),
            on_next(260, 5),
            on_next(290, 13),
            on_next(320, 3),
            on_completed(330),
            on_next(350, 7),
            on_next(390, 4),
            on_next(410, 17),
            on_next(450, 8),
            on_next(500, 23),
            on_completed(600),
        )
        invoked = 0

        def factory():
            def predicate(x):
                nonlocal invoked

                invoked += 1
                return is_prime(x)

            return xs.pipe(ops.take_while(predicate))

        results = scheduler.start(factory)

        assert results.messages == [
            on_next(210, 2),
            on_next(260, 5),
            on_next(290, 13),
            on_next(320, 3),
            on_completed(330),
        ]
        assert xs.subscriptions == [subscribe(200, 330)]
        assert invoked == 4

    def test_take_while_complete_after(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(90, -1),
            on_next(110, -1),
            on_next(210, 2),
            on_next(260, 5),
            on_next(290, 13),
            on_next(320, 3),
            on_next(350, 7),
            on_next(390, 4),
            on_next(410, 17),
            on_next(450, 8),
            on_next(500, 23),
            on_completed(600),
        )
        invoked = 0

        def factory():
            def predicate(x):
                nonlocal invoked

                invoked += 1
                return is_prime(x)

            return xs.pipe(ops.take_while(predicate))

        results = scheduler.start(factory)

        assert results.messages == [
            on_next(210, 2),
            on_next(260, 5),
            on_next(290, 13),
            on_next(320, 3),
            on_next(350, 7),
            on_completed(390),
        ]
        assert xs.subscriptions == [subscribe(200, 390)]
        assert invoked == 6

    def test_take_while_error_before(self):
        ex = "ex"
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(90, -1),
            on_next(110, -1),
            on_next(210, 2),
            on_next(260, 5),
            on_error(270, ex),
            on_next(290, 13),
            on_next(320, 3),
            on_next(350, 7),
            on_next(390, 4),
            on_next(410, 17),
            on_next(450, 8),
            on_next(500, 23),
        )
        invoked = 0

        def factory():
            def predicate(x):
                nonlocal invoked

                invoked += 1
                return is_prime(x)

            return xs.pipe(ops.take_while(predicate))

        results = scheduler.start(factory)

        assert results.messages == [on_next(210, 2), on_next(260, 5), on_error(270, ex)]
        assert xs.subscriptions == [subscribe(200, 270)]
        assert invoked == 2

    def test_take_while_error_after(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(90, -1),
            on_next(110, -1),
            on_next(210, 2),
            on_next(260, 5),
            on_next(290, 13),
            on_next(320, 3),
            on_next(350, 7),
            on_next(390, 4),
            on_next(410, 17),
            on_next(450, 8),
            on_next(500, 23),
            on_error(600, "ex"),
        )
        invoked = 0

        def factory():
            def predicate(x):
                nonlocal invoked

                invoked += 1
                return is_prime(x)

            return xs.pipe(ops.take_while(predicate))

        results = scheduler.start(factory)

        assert results.messages == [
            on_next(210, 2),
            on_next(260, 5),
            on_next(290, 13),
            on_next(320, 3),
            on_next(350, 7),
            on_completed(390),
        ]
        assert xs.subscriptions == [subscribe(200, 390)]
        assert invoked == 6

    def test_take_while_dispose_before(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(90, -1),
            on_next(110, -1),
            on_next(210, 2),
            on_next(260, 5),
            on_next(290, 13),
            on_next(320, 3),
            on_next(350, 7),
            on_next(390, 4),
            on_next(410, 17),
            on_next(450, 8),
            on_next(500, 23),
            on_completed(600),
        )
        invoked = 0

        def create():
            def predicate(x):
                nonlocal invoked

                invoked += 1
                return is_prime(x)

            return xs.pipe(ops.take_while(predicate))

        results = scheduler.start(create, disposed=300)
        assert results.messages == [on_next(210, 2), on_next(260, 5), on_next(290, 13)]
        assert xs.subscriptions == [subscribe(200, 300)]
        assert invoked == 3

    def test_take_while_dispose_after(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(90, -1),
            on_next(110, -1),
            on_next(210, 2),
            on_next(260, 5),
            on_next(290, 13),
            on_next(320, 3),
            on_next(350, 7),
            on_next(390, 4),
            on_next(410, 17),
            on_next(450, 8),
            on_next(500, 23),
            on_completed(600),
        )
        invoked = 0

        def create():
            def predicate(x):
                nonlocal invoked

                invoked += 1
                return is_prime(x)

            return xs.pipe(ops.take_while(predicate))

        results = scheduler.start(create, disposed=400)
        assert results.messages == [
            on_next(210, 2),
            on_next(260, 5),
            on_next(290, 13),
            on_next(320, 3),
            on_next(350, 7),
            on_completed(390),
        ]
        assert xs.subscriptions == [subscribe(200, 390)]
        assert invoked == 6

    def test_take_while_zero(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(90, -1),
            on_next(110, -1),
            on_next(205, 100),
            on_next(210, 2),
            on_next(260, 5),
            on_next(290, 13),
            on_next(320, 3),
            on_next(350, 7),
            on_next(390, 4),
            on_next(410, 17),
            on_next(450, 8),
            on_next(500, 23),
            on_completed(600),
        )
        invoked = 0

        def create():
            def predicate(x):
                nonlocal invoked

                invoked += 1
                return is_prime(x)

            return xs.pipe(ops.take_while(predicate))

        results = scheduler.start(create, disposed=300)
        assert results.messages == [on_completed(205)]
        assert xs.subscriptions == [subscribe(200, 205)]
        assert invoked == 1

    def test_take_while_on_error(self):
        ex = "ex"
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(90, -1),
            on_next(110, -1),
            on_next(210, 2),
            on_next(260, 5),
            on_next(290, 13),
            on_next(320, 3),
            on_next(350, 7),
            on_next(390, 4),
            on_next(410, 17),
            on_next(450, 8),
            on_next(500, 23),
            on_completed(600),
        )
        invoked = 0

        def factory():
            def predicate(x):
                nonlocal invoked

                invoked += 1
                if invoked == 3:
                    raise Exception(ex)

                return is_prime(x)

            return xs.pipe(ops.take_while(predicate))

        results = scheduler.start(factory)

        assert results.messages == [on_next(210, 2), on_next(260, 5), on_error(290, ex)]
        assert xs.subscriptions == [subscribe(200, 290)]
        assert invoked == 3

    def test_take_while_index(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(90, -1),
            on_next(110, -1),
            on_next(205, 100),
            on_next(210, 2),
            on_next(260, 5),
            on_next(290, 13),
            on_next(320, 3),
            on_next(350, 7),
            on_next(390, 4),
            on_next(410, 17),
            on_next(450, 8),
            on_next(500, 23),
            on_completed(600),
        )

        def factory():
            return xs.pipe(ops.take_while_indexed(lambda x, i: i < 5))

        results = scheduler.start(factory)

        assert results.messages == [
            on_next(205, 100),
            on_next(210, 2),
            on_next(260, 5),
            on_next(290, 13),
            on_next(320, 3),
            on_completed(350),
        ]
        assert xs.subscriptions == [subscribe(200, 350)]

    def test_take_while_index_inclusive_false(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(90, -1),
            on_next(110, -1),
            on_next(205, 100),
            on_next(210, 2),
            on_next(260, 5),
            on_next(290, 13),
            on_next(320, 3),
            on_next(350, 7),
            on_next(390, 4),
            on_next(410, 17),
            on_next(450, 8),
            on_next(500, 23),
            on_completed(600),
        )

        def factory():
            return xs.pipe(ops.take_while_indexed(lambda x, i: i < 5, inclusive=False))

        results = scheduler.start(factory)

        assert results.messages == [
            on_next(205, 100),
            on_next(210, 2),
            on_next(260, 5),
            on_next(290, 13),
            on_next(320, 3),
            on_completed(350),
        ]
        assert xs.subscriptions == [subscribe(200, 350)]

    def test_take_while_index_inclusive_true(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(90, -1),
            on_next(110, -1),
            on_next(205, 100),
            on_next(210, 2),
            on_next(260, 5),
            on_next(290, 13),
            on_next(320, 3),
            on_next(350, 7),
            on_next(390, 4),
            on_next(410, 17),
            on_next(450, 8),
            on_next(500, 23),
            on_completed(600),
        )

        def factory_inclusive():
            return xs.pipe(ops.take_while_indexed(lambda x, i: i < 4, inclusive=True))

        results_inclusive = scheduler.start(factory_inclusive)

        assert results_inclusive.messages == [
            on_next(205, 100),
            on_next(210, 2),
            on_next(260, 5),
            on_next(290, 13),
            on_next(320, 3),
            on_completed(320),
        ]
        assert xs.subscriptions == [subscribe(200, 320)]

    def test_take_while_complete_after_inclusive_true(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(90, -1),
            on_next(110, -1),
            on_next(210, 2),
            on_next(260, 5),
            on_next(290, 13),
            on_next(320, 3),
            on_next(350, 7),
            on_next(390, 4),
            on_next(410, 17),
            on_next(450, 8),
            on_next(500, 23),
            on_completed(600),
        )
        invoked = 0

        def factory():
            def predicate(x):
                nonlocal invoked

                invoked += 1
                return is_prime(x)

            return xs.pipe(ops.take_while(predicate, inclusive=True))

        results = scheduler.start(factory)

        assert results.messages == [
            on_next(210, 2),
            on_next(260, 5),
            on_next(290, 13),
            on_next(320, 3),
            on_next(350, 7),
            on_next(390, 4),
            on_completed(390),
        ]
        assert xs.subscriptions == [subscribe(200, 390)]
        assert invoked == 6



import unittest

from reactivex import operators as ops
from reactivex.testing import ReactiveTest, TestScheduler

on_next = ReactiveTest.on_next
on_completed = ReactiveTest.on_completed
on_error = ReactiveTest.on_error
subscribe = ReactiveTest.subscribe
subscribed = ReactiveTest.subscribed
disposed = ReactiveTest.disposed
created = ReactiveTest.created


class TestWindowWithTime(unittest.TestCase):
    def test_window_with_time_or_count_basic(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(205, 1),
            on_next(210, 2),
            on_next(240, 3),
            on_next(280, 4),
            on_next(320, 5),
            on_next(350, 6),
            on_next(370, 7),
            on_next(420, 8),
            on_next(470, 9),
            on_completed(600),
        )

        def create():
            def projection(w, i):
                def inner_proj(x):
                    return "%s %s" % (i, x)

                return w.pipe(ops.map(inner_proj))

            return xs.pipe(
                ops.window_with_time_or_count(70, 3),
                ops.map_indexed(projection),
                ops.merge_all(),
            )

        results = scheduler.start(create)
        assert results.messages == [
            on_next(205, "0 1"),
            on_next(210, "0 2"),
            on_next(240, "0 3"),
            on_next(280, "1 4"),
            on_next(320, "2 5"),
            on_next(350, "2 6"),
            on_next(370, "2 7"),
            on_next(420, "3 8"),
            on_next(470, "4 9"),
            on_completed(600),
        ]
        assert xs.subscriptions == [subscribe(200, 600)]

    def test_window_with_time_or_count_error(self):
        ex = "ex"
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(205, 1),
            on_next(210, 2),
            on_next(240, 3),
            on_next(280, 4),
            on_next(320, 5),
            on_next(350, 6),
            on_next(370, 7),
            on_next(420, 8),
            on_next(470, 9),
            on_error(600, ex),
        )

        def create():
            def projection(w, i):
                def inner_proj(x):
                    return "%s %s" % (i, x)

                return w.pipe(ops.map(inner_proj))

            return xs.pipe(
                ops.window_with_time_or_count(70, 3),
                ops.map_indexed(projection),
                ops.merge_all(),
            )

        results = scheduler.start(create)

        assert results.messages == [
            on_next(205, "0 1"),
            on_next(210, "0 2"),
            on_next(240, "0 3"),
            on_next(280, "1 4"),
            on_next(320, "2 5"),
            on_next(350, "2 6"),
            on_next(370, "2 7"),
            on_next(420, "3 8"),
            on_next(470, "4 9"),
            on_error(600, ex),
        ]
        assert xs.subscriptions == [subscribe(200, 600)]

    def test_window_with_time_or_count_disposed(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(205, 1),
            on_next(210, 2),
            on_next(240, 3),
            on_next(280, 4),
            on_next(320, 5),
            on_next(350, 6),
            on_next(370, 7),
            on_next(420, 8),
            on_next(470, 9),
            on_completed(600),
        )

        def create():
            def projection(w, i):
                def inner_proj(x):
                    return "%s %s" % (i, x)

                return w.pipe(ops.map(inner_proj))

            return xs.pipe(
                ops.window_with_time_or_count(70, 3),
                ops.map_indexed(projection),
                ops.merge_all(),
            )

        results = scheduler.start(create, disposed=370)
        assert results.messages == [
            on_next(205, "0 1"),
            on_next(210, "0 2"),
            on_next(240, "0 3"),
            on_next(280, "1 4"),
            on_next(320, "2 5"),
            on_next(350, "2 6"),
            on_next(370, "2 7"),
        ]
        assert xs.subscriptions == [subscribe(200, 370)]



from reactivex.disposable import (
    BooleanDisposable,
    CompositeDisposable,
    Disposable,
    RefCountDisposable,
    SerialDisposable,
    SingleAssignmentDisposable,
)


def test_Disposable_create():
    def action():
        pass

    disp = Disposable(action)
    assert disp


def test_Disposable_dispose():
    disposed = [False]

    def action():
        disposed[0] = True

    d = Disposable(action)
    assert not disposed[0]
    d.dispose()
    assert disposed[0]


def test_emptydisposable():
    d = Disposable()
    assert d
    d.dispose()


def test_booleandisposable():
    d = BooleanDisposable()
    assert not d.is_disposed
    d.dispose()
    assert d.is_disposed
    d.dispose()
    assert d.is_disposed


def test_future_disposable_setnone():
    d = SingleAssignmentDisposable()
    d.disposable = None
    assert d.disposable == None


def test_futuredisposable_disposeafterset():
    d = SingleAssignmentDisposable()
    disposed = [False]

    def action():
        disposed[0] = True

    dd = Disposable(action)
    d.disposable = dd
    assert dd == d.disposable
    assert not disposed[0]

    d.dispose()
    assert disposed[0]
    d.dispose()
    assert disposed[0]


def test_futuredisposable_disposebeforeset():
    disposed = [False]

    def dispose():
        disposed[0] = True

    d = SingleAssignmentDisposable()
    dd = Disposable(dispose)

    assert not disposed[0]
    d.dispose()
    assert not disposed[0]
    d.disposable = dd
    assert d.disposable == None
    assert disposed[0]
    d.dispose()
    assert disposed[0]


def test_groupdisposable_contains():
    d1 = Disposable()
    d2 = Disposable()

    g = CompositeDisposable(d1, d2)

    assert g.length == 2
    assert g.contains(d1)
    assert g.contains(d2)


def test_groupdisposable_add():
    d1 = Disposable()
    d2 = Disposable()

    g = CompositeDisposable(d1)

    assert g.length == 1
    assert g.contains(d1)
    g.add(d2)
    assert g.length == 2
    assert g.contains(d2)


def test_groupdisposable_addafterdispose():
    disp1 = [False]
    disp2 = [False]

    def action1():
        disp1[0] = True

    d1 = Disposable(action1)

    def action2():
        disp2[0] = True

    d2 = Disposable(action2)

    g = CompositeDisposable(d1)
    assert g.length == 1
    g.dispose()
    assert disp1[0]
    assert g.length == 0
    g.add(d2)
    assert disp2[0]
    assert g.length == 0


def test_groupdisposable_remove():
    disp1 = [False]
    disp2 = [False]

    def action1():
        disp1[0] = True

    d1 = Disposable(action1)

    def action2():
        disp2[0] = True

    d2 = Disposable(action2)

    g = CompositeDisposable(d1, d2)

    assert g.length == 2
    assert g.contains(d1)
    assert g.contains(d2)
    assert g.remove(d1)
    assert g.length == 1
    assert not g.contains(d1)
    assert g.contains(d2)
    assert disp1[0]
    assert g.remove(d2)
    assert not g.contains(d1)
    assert not g.contains(d2)
    assert disp2[0]

    disp3 = [False]

    def action3():
        disp3[0] = True

    d3 = Disposable(action3)
    assert not g.remove(d3)
    assert not disp3[0]


def test_groupdisposable_clear():
    disp1 = [False]
    disp2 = [False]

    def action1():
        disp1[0] = True

    d1 = Disposable(action1)

    def action2():
        disp2[0] = True

    d2 = Disposable(action2)

    g = CompositeDisposable(d1, d2)
    assert g.length == 2

    g.clear()
    assert disp1[0]
    assert disp2[0]
    assert not g.length

    disp3 = [False]

    def action3():
        disp3[0] = True

    d3 = Disposable(action3)
    g.add(d3)
    assert not disp3[0]
    assert g.length == 1


def test_mutabledisposable_ctor_prop():
    m = SerialDisposable()
    assert not m.disposable


def test_mutabledisposable_replacebeforedispose():
    disp1 = [False]
    disp2 = [False]
    m = SerialDisposable()

    def action1():
        disp1[0] = True

    d1 = Disposable(action1)
    m.disposable = d1

    assert d1 == m.disposable
    assert not disp1[0]

    def action2():
        disp2[0] = True

    d2 = Disposable(action2)
    m.disposable = d2

    assert d2 == m.disposable
    assert disp1[0]
    assert not disp2[0]


def test_mutabledisposable_replaceafterdispose():
    disp1 = [False]
    disp2 = [False]
    m = SerialDisposable()
    m.dispose()

    def action1():
        disp1[0] = True

    d1 = Disposable(action1)
    m.disposable = d1

    assert m.disposable == None
    assert disp1[0]

    def action2():
        disp2[0] = True

    d2 = Disposable(action2)
    m.disposable = d2

    assert m.disposable == None
    assert disp2[0]


def test_mutabledisposable_dispose():
    disp = [False]
    m = SerialDisposable()

    def action():
        disp[0] = True

    d = Disposable(action)
    m.disposable = d

    assert d == m.disposable
    assert not disp[0]
    m.dispose()
    assert disp[0]
    assert m.disposable == None


def test_refcountdisposable_singlereference():
    d = BooleanDisposable()
    r = RefCountDisposable(d)

    assert not d.is_disposed
    r.dispose()
    assert d.is_disposed
    r.dispose()
    assert d.is_disposed


def test_refcountdisposable_refcounting():
    d = BooleanDisposable()
    r = RefCountDisposable(d)
    assert not d.is_disposed
    d1 = r.disposable
    d2 = r.disposable
    assert not d.is_disposed
    d1.dispose()
    assert not d.is_disposed
    d2.dispose()
    assert not d.is_disposed
    r.dispose()
    assert d.is_disposed
    d3 = r.disposable
    d3.dispose()


def test_refcountdisposable_primarydisposesfirst():
    d = BooleanDisposable()
    r = RefCountDisposable(d)
    assert not d.is_disposed
    d1 = r.disposable
    d2 = r.disposable
    assert not d.is_disposed
    d1.dispose()
    assert not d.is_disposed
    r.dispose()
    assert not d.is_disposed
    d2.dispose()
    assert d.is_disposed



import asyncio
import os
import unittest
from datetime import datetime, timedelta, timezone

import pytest

from reactivex.scheduler.eventloop import AsyncIOScheduler

CI = os.getenv("CI") is not None


class TestAsyncIOScheduler(unittest.TestCase):
    @pytest.mark.skipif(CI, reason="Flaky test in GitHub Actions")
    def test_asyncio_schedule_now(self):
        loop = asyncio.get_event_loop()
        scheduler = AsyncIOScheduler(loop)
        diff = scheduler.now - datetime.fromtimestamp(loop.time(), tz=timezone.utc)
        assert abs(diff) < timedelta(milliseconds=2)  # NOTE: may take 1 ms in CI

    @pytest.mark.skipif(CI, reason="Test is flaky in GitHub Actions")
    def test_asyncio_schedule_now_units(self):
        loop = asyncio.get_event_loop()
        scheduler = AsyncIOScheduler(loop)
        diff = scheduler.now
        yield from asyncio.sleep(0.1)
        diff = scheduler.now - diff
        assert timedelta(milliseconds=80) < diff < timedelta(milliseconds=180)

    def test_asyncio_schedule_action(self):
        loop = asyncio.get_event_loop()

        async def go():
            scheduler = AsyncIOScheduler(loop)
            ran = False

            def action(scheduler, state):
                nonlocal ran
                ran = True

            scheduler.schedule(action)

            await asyncio.sleep(0.1)
            assert ran is True

        loop.run_until_complete(go())

    def test_asyncio_schedule_action_due(self):
        loop = asyncio.get_event_loop()

        async def go():
            scheduler = AsyncIOScheduler(loop)
            starttime = loop.time()
            endtime = None

            def action(scheduler, state):
                nonlocal endtime
                endtime = loop.time()

            scheduler.schedule_relative(0.2, action)

            await asyncio.sleep(0.3)
            assert endtime is not None
            diff = endtime - starttime
            assert diff > 0.18

        loop.run_until_complete(go())

    def test_asyncio_schedule_action_cancel(self):
        loop = asyncio.get_event_loop()

        async def go():
            ran = False
            scheduler = AsyncIOScheduler(loop)

            def action(scheduler, state):
                nonlocal ran
                ran = True

            d = scheduler.schedule_relative(0.05, action)
            d.dispose()

            await asyncio.sleep(0.3)
            assert ran is False

        loop.run_until_complete(go())



import unittest

from reactivex import operators as ops
from reactivex.testing import ReactiveTest, TestScheduler

on_next = ReactiveTest.on_next
on_completed = ReactiveTest.on_completed
on_error = ReactiveTest.on_error
subscribe = ReactiveTest.subscribe
subscribed = ReactiveTest.subscribed
disposed = ReactiveTest.disposed
created = ReactiveTest.created


class TestSequenceEqual(unittest.TestCase):
    def test_sequence_equal_equal(self):
        scheduler = TestScheduler()
        msgs1 = [
            on_next(110, 1),
            on_next(190, 2),
            on_next(240, 3),
            on_next(290, 4),
            on_next(310, 5),
            on_next(340, 6),
            on_next(450, 7),
            on_completed(510),
        ]
        msgs2 = [
            on_next(90, 1),
            on_next(270, 3),
            on_next(280, 4),
            on_next(300, 5),
            on_next(330, 6),
            on_next(340, 7),
            on_completed(720),
        ]
        xs = scheduler.create_hot_observable(msgs1)
        ys = scheduler.create_hot_observable(msgs2)
        results = scheduler.start(lambda: xs.pipe(ops.sequence_equal(ys)))

        assert results.messages == [on_next(720, True), on_completed(720)]
        assert xs.subscriptions == [subscribe(200, 510)]
        assert ys.subscriptions == [subscribe(200, 720)]

    def test_sequence_equal_equal_sym(self):
        scheduler = TestScheduler()
        msgs1 = [
            on_next(110, 1),
            on_next(190, 2),
            on_next(240, 3),
            on_next(290, 4),
            on_next(310, 5),
            on_next(340, 6),
            on_next(450, 7),
            on_completed(510),
        ]
        msgs2 = [
            on_next(90, 1),
            on_next(270, 3),
            on_next(280, 4),
            on_next(300, 5),
            on_next(330, 6),
            on_next(340, 7),
            on_completed(720),
        ]
        xs = scheduler.create_hot_observable(msgs1)
        ys = scheduler.create_hot_observable(msgs2)
        results = scheduler.start(lambda: ys.pipe(ops.sequence_equal(xs)))

        assert results.messages == [on_next(720, True), on_completed(720)]
        assert xs.subscriptions == [subscribe(200, 510)]
        assert ys.subscriptions == [subscribe(200, 720)]

    def test_sequence_equal_not_equal_left(self):
        scheduler = TestScheduler()
        msgs1 = [
            on_next(110, 1),
            on_next(190, 2),
            on_next(240, 3),
            on_next(290, 4),
            on_next(310, 0),
            on_next(340, 6),
            on_next(450, 7),
            on_completed(510),
        ]
        msgs2 = [
            on_next(90, 1),
            on_next(270, 3),
            on_next(280, 4),
            on_next(300, 5),
            on_next(330, 6),
            on_next(340, 7),
            on_completed(720),
        ]
        xs = scheduler.create_hot_observable(msgs1)
        ys = scheduler.create_hot_observable(msgs2)
        results = scheduler.start(lambda: xs.pipe(ops.sequence_equal(ys)))

        assert results.messages == [on_next(310, False), on_completed(310)]
        assert xs.subscriptions == [subscribe(200, 310)]
        assert ys.subscriptions == [subscribe(200, 310)]

    def test_sequence_equal_not_equal_left_sym(self):
        scheduler = TestScheduler()
        msgs1 = [
            on_next(110, 1),
            on_next(190, 2),
            on_next(240, 3),
            on_next(290, 4),
            on_next(310, 0),
            on_next(340, 6),
            on_next(450, 7),
            on_completed(510),
        ]
        msgs2 = [
            on_next(90, 1),
            on_next(270, 3),
            on_next(280, 4),
            on_next(300, 5),
            on_next(330, 6),
            on_next(340, 7),
            on_completed(720),
        ]
        xs = scheduler.create_hot_observable(msgs1)
        ys = scheduler.create_hot_observable(msgs2)
        results = scheduler.start(lambda: ys.pipe(ops.sequence_equal(xs)))

        assert results.messages == [on_next(310, False), on_completed(310)]
        assert xs.subscriptions == [subscribe(200, 310)]
        assert ys.subscriptions == [subscribe(200, 310)]

    def test_sequence_equal_not_equal_right(self):
        scheduler = TestScheduler()
        msgs1 = [
            on_next(110, 1),
            on_next(190, 2),
            on_next(240, 3),
            on_next(290, 4),
            on_next(310, 5),
            on_next(340, 6),
            on_next(450, 7),
            on_completed(510),
        ]
        msgs2 = [
            on_next(90, 1),
            on_next(270, 3),
            on_next(280, 4),
            on_next(300, 5),
            on_next(330, 6),
            on_next(340, 7),
            on_next(350, 8),
        ]
        xs = scheduler.create_hot_observable(msgs1)
        ys = scheduler.create_hot_observable(msgs2)
        results = scheduler.start(lambda: xs.pipe(ops.sequence_equal(ys)))

        assert results.messages == [on_next(510, False), on_completed(510)]
        assert xs.subscriptions == [subscribe(200, 510)]
        assert ys.subscriptions == [subscribe(200, 510)]

    def test_sequence_equal_not_equal_right_sym(self):
        scheduler = TestScheduler()
        msgs1 = [
            on_next(110, 1),
            on_next(190, 2),
            on_next(240, 3),
            on_next(290, 4),
            on_next(310, 5),
            on_next(340, 6),
            on_next(450, 7),
            on_completed(510),
        ]
        msgs2 = [
            on_next(90, 1),
            on_next(270, 3),
            on_next(280, 4),
            on_next(300, 5),
            on_next(330, 6),
            on_next(340, 7),
            on_next(350, 8),
        ]
        xs = scheduler.create_hot_observable(msgs1)
        ys = scheduler.create_hot_observable(msgs2)
        results = scheduler.start(lambda: ys.pipe(ops.sequence_equal(xs)))

        assert results.messages == [on_next(510, False), on_completed(510)]
        assert xs.subscriptions == [subscribe(200, 510)]
        assert ys.subscriptions == [subscribe(200, 510)]

    def test_sequence_equal_not_equal_2(self):
        scheduler = TestScheduler()
        msgs1 = [
            on_next(110, 1),
            on_next(190, 2),
            on_next(240, 3),
            on_next(290, 4),
            on_next(310, 5),
            on_next(340, 6),
            on_next(450, 7),
            on_next(490, 8),
            on_next(520, 9),
            on_next(580, 10),
            on_next(600, 11),
        ]
        msgs2 = [
            on_next(90, 1),
            on_next(270, 3),
            on_next(280, 4),
            on_next(300, 5),
            on_next(330, 6),
            on_next(340, 7),
            on_next(350, 9),
            on_next(400, 9),
            on_next(410, 10),
            on_next(490, 11),
            on_next(550, 12),
            on_next(560, 13),
        ]
        xs = scheduler.create_hot_observable(msgs1)
        ys = scheduler.create_hot_observable(msgs2)
        results = scheduler.start(create=lambda: xs.pipe(ops.sequence_equal(ys)))

        assert results.messages == [on_next(490, False), on_completed(490)]
        assert xs.subscriptions == [subscribe(200, 490)]
        assert ys.subscriptions == [subscribe(200, 490)]

    def test_sequence_equal_not_equal_2_sym(self):
        scheduler = TestScheduler()
        msgs1 = [
            on_next(110, 1),
            on_next(190, 2),
            on_next(240, 3),
            on_next(290, 4),
            on_next(310, 5),
            on_next(340, 6),
            on_next(450, 7),
            on_next(490, 8),
            on_next(520, 9),
            on_next(580, 10),
            on_next(600, 11),
        ]
        msgs2 = [
            on_next(90, 1),
            on_next(270, 3),
            on_next(280, 4),
            on_next(300, 5),
            on_next(330, 6),
            on_next(340, 7),
            on_next(350, 9),
            on_next(400, 9),
            on_next(410, 10),
            on_next(490, 11),
            on_next(550, 12),
            on_next(560, 13),
        ]
        xs = scheduler.create_hot_observable(msgs1)
        ys = scheduler.create_hot_observable(msgs2)
        results = scheduler.start(create=lambda: ys.pipe(ops.sequence_equal(xs)))

        assert results.messages == [on_next(490, False), on_completed(490)]
        assert xs.subscriptions == [subscribe(200, 490)]
        assert ys.subscriptions == [subscribe(200, 490)]

    def test_sequence_equal_not_equal_3(self):
        scheduler = TestScheduler()
        msgs1 = [
            on_next(110, 1),
            on_next(190, 2),
            on_next(240, 3),
            on_next(290, 4),
            on_next(310, 5),
            on_completed(330),
        ]
        msgs2 = [on_next(90, 1), on_next(270, 3), on_next(400, 4), on_completed(420)]
        xs = scheduler.create_hot_observable(msgs1)
        ys = scheduler.create_hot_observable(msgs2)
        results = scheduler.start(create=lambda: xs.pipe(ops.sequence_equal(ys)))

        assert results.messages == [on_next(420, False), on_completed(420)]
        assert xs.subscriptions == [subscribe(200, 330)]
        assert ys.subscriptions == [subscribe(200, 420)]

    def test_sequence_equal_not_equal_3_sym(self):
        scheduler = TestScheduler()
        msgs1 = [
            on_next(110, 1),
            on_next(190, 2),
            on_next(240, 3),
            on_next(290, 4),
            on_next(310, 5),
            on_completed(330),
        ]
        msgs2 = [on_next(90, 1), on_next(270, 3), on_next(400, 4), on_completed(420)]
        xs = scheduler.create_hot_observable(msgs1)
        ys = scheduler.create_hot_observable(msgs2)
        results = scheduler.start(create=lambda: ys.pipe(ops.sequence_equal(xs)))

        assert results.messages == [on_next(420, False), on_completed(420)]
        assert xs.subscriptions == [subscribe(200, 330)]
        assert ys.subscriptions == [subscribe(200, 420)]

    def test_sequence_equal_comparer_throws(self):
        ex = "ex"
        scheduler = TestScheduler()
        msgs1 = [
            on_next(110, 1),
            on_next(190, 2),
            on_next(240, 3),
            on_next(290, 4),
            on_next(310, 5),
            on_completed(330),
        ]
        msgs2 = [on_next(90, 1), on_next(270, 3), on_next(400, 4), on_completed(420)]
        xs = scheduler.create_hot_observable(msgs1)
        ys = scheduler.create_hot_observable(msgs2)

        def create():
            def comparer(a, b):
                raise Exception(ex)

            return xs.pipe(ops.sequence_equal(ys, comparer))

        results = scheduler.start(create=create)

        assert results.messages == [on_error(270, ex)]
        assert xs.subscriptions == [subscribe(200, 270)]
        assert ys.subscriptions == [subscribe(200, 270)]

    def test_sequence_equal_comparer_throws_sym(self):
        ex = "ex"
        scheduler = TestScheduler()
        msgs1 = [
            on_next(110, 1),
            on_next(190, 2),
            on_next(240, 3),
            on_next(290, 4),
            on_next(310, 5),
            on_completed(330),
        ]
        msgs2 = [on_next(90, 1), on_next(270, 3), on_next(400, 4), on_completed(420)]
        xs = scheduler.create_hot_observable(msgs1)
        ys = scheduler.create_hot_observable(msgs2)

        def create():
            def comparer(a, b):
                raise Exception(ex)

            return ys.pipe(ops.sequence_equal(xs, comparer))

        results = scheduler.start(create=create)

        assert results.messages == [on_error(270, ex)]
        assert xs.subscriptions == [subscribe(200, 270)]
        assert ys.subscriptions == [subscribe(200, 270)]

    def test_sequence_equal_not_equal_4(self):
        scheduler = TestScheduler()
        msgs1 = [on_next(250, 1), on_completed(300)]
        msgs2 = [on_next(290, 1), on_next(310, 2), on_completed(350)]
        xs = scheduler.create_hot_observable(msgs1)
        ys = scheduler.create_hot_observable(msgs2)
        results = scheduler.start(create=lambda: xs.pipe(ops.sequence_equal(ys)))

        assert results.messages == [on_next(310, False), on_completed(310)]
        assert xs.subscriptions == [subscribe(200, 300)]
        assert ys.subscriptions == [subscribe(200, 310)]

    def test_sequence_equal_not_equal_4_sym(self):
        scheduler = TestScheduler()
        msgs1 = [on_next(250, 1), on_completed(300)]
        msgs2 = [on_next(290, 1), on_next(310, 2), on_completed(350)]
        xs = scheduler.create_hot_observable(msgs1)
        ys = scheduler.create_hot_observable(msgs2)
        results = scheduler.start(create=lambda: ys.pipe(ops.sequence_equal(xs)))

        assert results.messages == [on_next(310, False), on_completed(310)]
        assert xs.subscriptions == [subscribe(200, 300)]
        assert ys.subscriptions == [subscribe(200, 310)]

    def test_sequenceequal_iterable_equal(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(110, 1),
            on_next(190, 2),
            on_next(240, 3),
            on_next(290, 4),
            on_next(310, 5),
            on_next(340, 6),
            on_next(450, 7),
            on_completed(510),
        )

        def create():
            return xs.pipe(ops.sequence_equal([3, 4, 5, 6, 7]))

        res = scheduler.start(create=create)

        assert res.messages == [on_next(510, True), on_completed(510)]
        assert xs.subscriptions == [subscribe(200, 510)]

    def test_sequenceequal_iterable_notequal_elements(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(110, 1),
            on_next(190, 2),
            on_next(240, 3),
            on_next(290, 4),
            on_next(310, 5),
            on_next(340, 6),
            on_next(450, 7),
            on_completed(510),
        )

        def create():
            return xs.pipe(ops.sequence_equal([3, 4, 9, 6, 7]))

        res = scheduler.start(create=create)

        assert res.messages == [on_next(310, False), on_completed(310)]
        assert xs.subscriptions == [subscribe(200, 310)]

    def test_sequenceequal_iterable_comparer_equal(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(110, 1),
            on_next(190, 2),
            on_next(240, 3),
            on_next(290, 4),
            on_next(310, 5),
            on_next(340, 6),
            on_next(450, 7),
            on_completed(510),
        )

        def create():
            def comparer(x, y):
                return x % 2 == y % 2

            return xs.pipe(ops.sequence_equal([3 - 2, 4, 5, 6 + 42, 7 - 6], comparer))

        res = scheduler.start(create=create)

        assert res.messages == [on_next(510, True), on_completed(510)]
        assert xs.subscriptions == [subscribe(200, 510)]

    def test_sequenceequal_iterable_comparer_notequal(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(110, 1),
            on_next(190, 2),
            on_next(240, 3),
            on_next(290, 4),
            on_next(310, 5),
            on_next(340, 6),
            on_next(450, 7),
            on_completed(510),
        )

        def create():
            def comparer(x, y):
                return x % 2 == y % 2

            return xs.pipe(
                ops.sequence_equal([3 - 2, 4, 5 + 9, 6 + 42, 7 - 6], comparer)
            )

        res = scheduler.start(create=create)

        assert res.messages == [on_next(310, False), on_completed(310)]
        assert xs.subscriptions == [subscribe(200, 310)]

    def test_sequenceequal_iterable_comparer_throws(self):
        def on_error_comparer(value, exn):
            def comparer(x, y):
                if x == value:
                    raise Exception(exn)

                return x == y

            return comparer

        ex = "ex"
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(110, 1),
            on_next(190, 2),
            on_next(240, 3),
            on_next(290, 4),
            on_next(310, 5),
            on_next(340, 6),
            on_next(450, 7),
            on_completed(510),
        )

        def create():
            return xs.pipe(
                ops.sequence_equal([3, 4, 5, 6, 7], on_error_comparer(5, ex))
            )

        res = scheduler.start(create=create)

        assert res.messages == [on_error(310, ex)]
        assert xs.subscriptions == [subscribe(200, 310)]

    def test_sequenceequal_iterable_notequal_toolong(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(110, 1),
            on_next(190, 2),
            on_next(240, 3),
            on_next(290, 4),
            on_next(310, 5),
            on_next(340, 6),
            on_next(450, 7),
            on_completed(510),
        )

        def create():
            return xs.pipe(ops.sequence_equal([3, 4, 5, 6, 7, 8]))

        res = scheduler.start(create=create)

        assert res.messages == [on_next(510, False), on_completed(510)]
        assert xs.subscriptions == [subscribe(200, 510)]

    def test_sequenceequal_iterable_notequal_tooshort(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(110, 1),
            on_next(190, 2),
            on_next(240, 3),
            on_next(290, 4),
            on_next(310, 5),
            on_next(340, 6),
            on_next(450, 7),
            on_completed(510),
        )

        def create():
            return xs.pipe(ops.sequence_equal([3, 4, 5, 6]))

        res = scheduler.start(create=create)

        assert res.messages == [on_next(450, False), on_completed(450)]
        assert xs.subscriptions == [subscribe(200, 450)]

    def test_sequenceequal_iterable_on_error(self):
        ex = "ex"
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(110, 1),
            on_next(190, 2),
            on_next(240, 3),
            on_next(290, 4),
            on_error(310, ex),
        )

        def create():
            return xs.pipe(ops.sequence_equal([3, 4]))

        res = scheduler.start(create=create)

        assert res.messages == [on_error(310, ex)]
        assert xs.subscriptions == [subscribe(200, 310)]



import reactivex
from reactivex import operators as ops

a = reactivex.cold(" ---a0---a1----------------a2-|    ")
b = reactivex.cold("    ---b1---b2---|                 ")
c = reactivex.cold("             ---c1---c2---|        ")
d = reactivex.cold("                   -----d1---d2---|")
e1 = reactivex.cold("a--b--------c-----d-------|       ")

observableLookup = {"a": a, "b": b, "c": c, "d": d}

source = e1.pipe(
    ops.flat_map(lambda value: observableLookup[value]),
    ops.do_action(lambda v: print(v)),
)

source.run()



import unittest

from reactivex import interval
from reactivex import operators as ops
from reactivex.testing import ReactiveTest, TestScheduler
from reactivex.testing.marbles import marbles_testing
from reactivex.testing.subscription import Subscription

on_next = ReactiveTest.on_next
on_completed = ReactiveTest.on_completed
on_error = ReactiveTest.on_error
subscribe = ReactiveTest.subscribe
subscribed = ReactiveTest.subscribed
disposed = ReactiveTest.disposed
created = ReactiveTest.created


class TestSwitchMap(unittest.TestCase):
    def test_switch_map(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(300, "a"),
            on_next(400, "b"),
            on_next(500, "c"),
        )

        def create_inner(x: str):
            def create_changing(j: int):
                return (j, x)

            return interval(20).pipe(ops.map(create_changing))

        def create():
            return xs.pipe(ops.switch_map(project=create_inner))

        results = scheduler.start(create, disposed=580)
        # (i, j, x): i is the index of the outer emit;
        # j is the value of the inner interval;
        # x is the value of the outer emission
        assert results.messages == [
            on_next(320, (0, "a")),
            on_next(340, (1, "a")),
            on_next(360, (2, "a")),
            on_next(380, (3, "a")),
            on_next(420, (0, "b")),
            on_next(440, (1, "b")),
            on_next(460, (2, "b")),
            on_next(480, (3, "b")),
            on_next(520, (0, "c")),
            on_next(540, (1, "c")),
            on_next(560, (2, "c")),
        ]
        assert xs.subscriptions == [Subscription(200, 580)]

    def test_switch_map_inner_throws(self):
        """Inner throwing causes outer to throw"""
        ex = "ex"
        scheduler = TestScheduler()
        sources = [
            scheduler.create_cold_observable(on_next(100, "a"), on_next(300, "aa")),
            scheduler.create_cold_observable(on_next(50, "b"), on_error(120, ex)),
            scheduler.create_cold_observable(
                on_next(50, "wont happen"), on_error(120, "no")
            ),
        ]
        xs = scheduler.create_hot_observable(
            on_next(
                250,
                0,
            ),
            on_next(400, 1),
            on_next(
                550,
                2,
            ),
        )

        def create_inner(x: int):
            return sources[x]

        def create():
            return xs.pipe(ops.switch_map(create_inner))

        results = scheduler.start(create)
        assert results.messages == [
            on_next(350, "a"),
            on_next(450, "b"),
            on_error(520, ex),
        ]
        assert sources[0].subscriptions == [Subscription(250, 400)]
        assert sources[1].subscriptions == [Subscription(400, 520)]
        assert sources[2].subscriptions == []

    def test_switch_map_outer_throws(self):
        """Outer throwing unsubscribes from all"""
        ex = "ABC"
        scheduler = TestScheduler()
        sources = [
            scheduler.create_cold_observable(on_next(100, "a"), on_next(300, "aa")),
            scheduler.create_cold_observable(on_next(50, "b"), on_error(120, ex)),
            scheduler.create_cold_observable(
                on_next(50, "wont happen"), on_error(120, "no")
            ),
        ]
        xs = scheduler.create_hot_observable(
            on_next(
                250,
                0,
            ),
            on_next(400, 1),
            on_error(430, ex),
        )

        def create_inner(x: int):
            return sources[x]

        def create():
            return xs.pipe(ops.switch_map(create_inner))

        results = scheduler.start(create)
        assert results.messages == [
            on_next(350, "a"),
            on_error(430, ex),
        ]
        assert sources[0].subscriptions == [Subscription(250, 400)]
        assert sources[1].subscriptions == [Subscription(400, 430)]
        assert sources[2].subscriptions == []

    def test_switch_map_no_inner(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(on_completed(500))
        # Fake inner which should never be subscribed to
        sources = [scheduler.create_cold_observable(on_next(20, 2))]

        def create_inner(_x: int):
            return sources.pop(0)

        def create():
            return xs.pipe(ops.switch_map(create_inner))

        results = scheduler.start(create)
        assert results.messages == [on_completed(500)]
        assert xs.subscriptions == [Subscription(200, 500)]
        assert sources[0].subscriptions == []

    def test_switch_map_inner_completes(self):
        """Inner completions do not affect outer"""
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(300, "d"),
            on_next(330, "f"),
            on_completed(540),
        )

        def create_inner(x: str):
            """An observable which will complete after 40 ticks"""
            return interval(20).pipe(ops.map(lambda j: (j, x)), ops.take(2))

        def create():
            return xs.pipe(ops.switch_map(create_inner))

        results = scheduler.start(create)
        assert results.messages == [
            on_next(320, (0, "d")),
            on_next(350, (0, "f")),
            on_next(
                370, (1, "f")
            ),  # here the current inner is unsubscribed but not the outer
            on_completed(540),  # only outer completion affects
        ]

    def test_switch_map_default_mapper(self):
        with marbles_testing(timespan=10) as (start, cold, hot, exp):
            xs = hot(
                "               ---a---b------c-----",
                {
                    "a": cold("    --1--2", None, None),
                    "b": cold("        --1-2-3-4-5|", None, None),
                    "c": cold("               --1--2", None, None),
                },
                None,
            )
            expected = exp("    -----1---1-2-3--1--2", None, None)
            result = start(xs.pipe(ops.switch_map()))
            assert result == expected



import unittest

from reactivex import operators as ops
from reactivex.testing import ReactiveTest, TestScheduler

on_next = ReactiveTest.on_next
on_completed = ReactiveTest.on_completed
on_error = ReactiveTest.on_error
subscribe = ReactiveTest.subscribe
subscribed = ReactiveTest.subscribed
disposed = ReactiveTest.disposed
created = ReactiveTest.created


class TestSkipWithTime(unittest.TestCase):
    def test_skip_zero(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(210, 1), on_next(220, 2), on_completed(230)
        )

        def create():
            return xs.pipe(ops.skip_with_time(0))

        res = scheduler.start(create)

        assert res.messages == [on_next(210, 1), on_next(220, 2), on_completed(230)]
        assert xs.subscriptions == [subscribe(200, 230)]

    def test_skip_some(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(210, 1), on_next(220, 2), on_completed(230)
        )

        def create():
            return xs.pipe(ops.skip_with_time(15))

        res = scheduler.start(create)

        assert res.messages == [on_next(220, 2), on_completed(230)]
        assert xs.subscriptions == [subscribe(200, 230)]

    def test_skip_late(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(210, 1), on_next(220, 2), on_completed(230)
        )

        def create():
            return xs.pipe(ops.skip_with_time(50))

        res = scheduler.start(create)

        assert res.messages == [on_completed(230)]
        assert xs.subscriptions == [subscribe(200, 230)]

    def test_skip_error(self):
        ex = "ex"
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(on_error(210, ex))

        def create():
            return xs.pipe(ops.skip_with_time(50))

        res = scheduler.start(create)

        assert res.messages == [on_error(210, ex)]
        assert xs.subscriptions == [subscribe(200, 210)]

    def test_skip_never(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable()

        def create():
            return xs.pipe(ops.skip_with_time(50))

        res = scheduler.start(create)

        assert res.messages == []
        assert xs.subscriptions == [subscribe(200, 1000)]

    def test_skip_twice1(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(210, 1),
            on_next(220, 2),
            on_next(230, 3),
            on_next(240, 4),
            on_next(250, 5),
            on_next(260, 6),
            on_completed(270),
        )

        def create():
            return xs.pipe(
                ops.skip_with_time(15),
                ops.skip_with_time(30),
            )

        res = scheduler.start(create)

        assert res.messages == [
            on_next(240, 4),
            on_next(250, 5),
            on_next(260, 6),
            on_completed(270),
        ]
        assert xs.subscriptions == [subscribe(200, 270)]

    def test_skip_twice2(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(210, 1),
            on_next(220, 2),
            on_next(230, 3),
            on_next(240, 4),
            on_next(250, 5),
            on_next(260, 6),
            on_completed(270),
        )

        def create():
            return xs.pipe(
                ops.skip_with_time(30),
                ops.skip_with_time(15),
            )

        res = scheduler.start(create)

        assert res.messages == [
            on_next(240, 4),
            on_next(250, 5),
            on_next(260, 6),
            on_completed(270),
        ]
        assert xs.subscriptions == [subscribe(200, 270)]



import reactivex
from reactivex import operators as ops

"""
Specify the error to be raised in place of the # symbol.
"""

err = ValueError("I don't like 5!")

src0 = reactivex.from_marbles("12-----4-----67--|", timespan=0.2)
src1 = reactivex.from_marbles("----3----5-#      ", timespan=0.2, error=err)

source = reactivex.merge(src0, src1).pipe(ops.do_action(print))
source.run()



import unittest

from reactivex import operators as ops
from reactivex.testing import ReactiveTest, TestScheduler

on_next = ReactiveTest.on_next
on_completed = ReactiveTest.on_completed
on_error = ReactiveTest.on_error
subscribe = ReactiveTest.subscribe
subscribed = ReactiveTest.subscribed
disposed = ReactiveTest.disposed
created = ReactiveTest.created


class TestSwitch(unittest.TestCase):
    def test_switch_data(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(
                300,
                scheduler.create_cold_observable(
                    on_next(10, 101),
                    on_next(20, 102),
                    on_next(110, 103),
                    on_next(120, 104),
                    on_next(210, 105),
                    on_next(220, 106),
                    on_completed(230),
                ),
            ),
            on_next(
                400,
                scheduler.create_cold_observable(
                    on_next(10, 201),
                    on_next(20, 202),
                    on_next(30, 203),
                    on_next(40, 204),
                    on_completed(50),
                ),
            ),
            on_next(
                500,
                scheduler.create_cold_observable(
                    on_next(10, 301),
                    on_next(20, 302),
                    on_next(30, 303),
                    on_next(40, 304),
                    on_completed(150),
                ),
            ),
            on_completed(600),
        )

        def create():
            return xs.pipe(ops.switch_latest())

        results = scheduler.start(create)
        assert results.messages == [
            on_next(310, 101),
            on_next(320, 102),
            on_next(410, 201),
            on_next(420, 202),
            on_next(430, 203),
            on_next(440, 204),
            on_next(510, 301),
            on_next(520, 302),
            on_next(530, 303),
            on_next(540, 304),
            on_completed(650),
        ]

    def test_switch_inner_throws(self):
        ex = "ex"
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(
                300,
                scheduler.create_cold_observable(
                    on_next(10, 101),
                    on_next(20, 102),
                    on_next(110, 103),
                    on_next(120, 104),
                    on_next(210, 105),
                    on_next(220, 106),
                    on_completed(230),
                ),
            ),
            on_next(
                400,
                scheduler.create_cold_observable(
                    on_next(10, 201),
                    on_next(20, 202),
                    on_next(30, 203),
                    on_next(40, 204),
                    on_error(50, ex),
                ),
            ),
            on_next(
                500,
                scheduler.create_cold_observable(
                    on_next(10, 301),
                    on_next(20, 302),
                    on_next(30, 303),
                    on_next(40, 304),
                    on_completed(150),
                ),
            ),
            on_completed(600),
        )

        def create():
            return xs.pipe(ops.switch_latest())

        results = scheduler.start(create)
        assert results.messages == [
            on_next(310, 101),
            on_next(320, 102),
            on_next(410, 201),
            on_next(420, 202),
            on_next(430, 203),
            on_next(440, 204),
            on_error(450, ex),
        ]

    def test_switch_outer_throws(self):
        ex = "ex"
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(
                300,
                scheduler.create_cold_observable(
                    on_next(10, 101),
                    on_next(20, 102),
                    on_next(110, 103),
                    on_next(120, 104),
                    on_next(210, 105),
                    on_next(220, 106),
                    on_completed(230),
                ),
            ),
            on_next(
                400,
                scheduler.create_cold_observable(
                    on_next(10, 201),
                    on_next(20, 202),
                    on_next(30, 203),
                    on_next(40, 204),
                    on_completed(50),
                ),
            ),
            on_error(500, ex),
        )

        def create():
            return xs.pipe(ops.switch_latest())

        results = scheduler.start(create)
        assert results.messages == [
            on_next(310, 101),
            on_next(320, 102),
            on_next(410, 201),
            on_next(420, 202),
            on_next(430, 203),
            on_next(440, 204),
            on_error(500, ex),
        ]

    def test_switch_no_inner(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(on_completed(500))

        def create():
            return xs.pipe(ops.switch_latest())

        results = scheduler.start(create)
        assert results.messages == [on_completed(500)]

    def test_switch_inner_completes(self):
        scheduler = TestScheduler()
        xs = scheduler.create_hot_observable(
            on_next(
                300,
                scheduler.create_cold_observable(
                    on_next(10, 101),
                    on_next(20, 102),
                    on_next(110, 103),
                    on_next(120, 104),
                    on_next(210, 105),
                    on_next(220, 106),
                    on_completed(230),
                ),
            ),
            on_completed(540),
        )

        def create():
            return xs.pipe(ops.switch_latest())

        results = scheduler.start(create)
        assert results.messages == [
            on_next(310, 101),
            on_next(320, 102),
            on_next(410, 103),
            on_next(420, 104),
            on_next(510, 105),
            on_next(520, 106),
            on_completed(540),
        ]



import unittest

from reactivex import empty
from reactivex.testing import ReactiveTest, TestScheduler

on_next = ReactiveTest.on_next
on_completed = ReactiveTest.on_completed
on_error = ReactiveTest.on_error
subscribe = ReactiveTest.subscribe
subscribed = ReactiveTest.subscribed
disposed = ReactiveTest.disposed
created = ReactiveTest.created


class RxException(Exception):
    pass


# Helper function for raising exceptions within lambdas
def _raise(ex):
    raise RxException(ex)


class TestEmpty(unittest.TestCase):
    def test_empty_basic(self):
        scheduler = TestScheduler()

        def factory():
            return empty()

        results = scheduler.start(factory)

        assert results.messages == [on_completed(200)]

    def test_empty_disposed(self):
        scheduler = TestScheduler()

        def factory():
            return empty()

        results = scheduler.start(factory, disposed=200)
        assert results.messages == []

    def test_empty_observer_throw_exception(self):
        scheduler = TestScheduler()
        xs = empty()
        xs.subscribe(
            lambda x: None, lambda ex: None, lambda: _raise("ex"), scheduler=scheduler
        )

        with self.assertRaises(RxException):
            scheduler.start()



import unittest

from reactivex import operators as ops
from reactivex.testing import ReactiveTest, TestScheduler

on_next = ReactiveTest.on_next
on_completed = ReactiveTest.on_completed
on_error = ReactiveTest.on_error
subscribe = ReactiveTest.subscribe
subscribed = ReactiveTest.subscribed
disposed = ReactiveTest.disposed
created = ReactiveTest.created


class TestPluck(unittest.TestCase):
    def test_pluck_completed(self):
        scheduler = TestScheduler()

        xs = scheduler.create_hot_observable(
            on_next(180, {"prop": 1}),
            on_next(210, {"prop": 2}),
            on_next(240, {"prop": 3}),
            on_next(290, {"prop": 4}),
            on_next(350, {"prop": 5}),
            on_completed(400),
            on_next(410, {"prop": -1}),
            on_completed(420),
            on_error(430, Exception("ex")),
        )
        results = scheduler.start(create=lambda: xs.pipe(ops.pluck("prop")))

        assert results.messages == [
            on_next(210, 2),
            on_next(240, 3),
            on_next(290, 4),
            on_next(350, 5),
            on_completed(400),
        ]
        assert xs.subscriptions == [subscribe(200, 400)]


class TestPluckAttr(unittest.TestCase):
    def test_pluck_attr_completed(self):
        scheduler = TestScheduler()

        class DummyClass:
            def __init__(self, prop):
                self.prop = prop

        xs = scheduler.create_hot_observable(
            on_next(180, DummyClass(1)),
            on_next(210, DummyClass(2)),
            on_next(240, DummyClass(3)),
            on_next(290, DummyClass(4)),
            on_next(350, DummyClass(5)),
            on_completed(400),
            on_next(410, DummyClass(-1)),
            on_completed(420),
            on_error(430, Exception("ex")),
        )
        results = scheduler.start(create=lambda: xs.pipe(ops.pluck_attr("prop")))

        assert results.messages == [
            on_next(210, 2),
            on_next(240, 3),
            on_next(290, 4),
            on_next(350, 5),
            on_completed(400),
        ]
        assert xs.subscriptions == [subscribe(200, 400)]
